"""card_group_actions

Revision: 0111
Revision ID: 54c1c3cc20fd
Revises: 1b7722f7b357
Create Date: 2018-01-31 16:40:54.911871

"""

# revision identifiers, used by Alembic.
revision = '54c1c3cc20fd'
down_revision = '1b7722f7b357'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.create_table('card_group_actions',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('group_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('action_name', sa.String(), nullable=True),
        sa.Column('options', postgresql.JSONB(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['group_id'], ['card_groups.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_group_actions_group_id'), 'card_group_actions', ['group_id'], unique=False)

    op.execute("""
        INSERT INTO card_group_actions (group_id, created_at, action_name, options)
        SELECT id, updated_at, 'set_attribute', ('{"attr": {"name": "' || auto_assign_user || '", "type": "members", "value": "@me"}}')::jsonb
        FROM card_groups WHERE auto_assign_user is not null
    """)


def downgrade():
    op.drop_index(op.f('ix_card_group_actions_group_id'), table_name='card_group_actions')
    op.drop_table('card_group_actions')



"""user_created_by

Revision: 0178
Revision ID: 3c5285bbbb66
Revises: 23727a96713b
Create Date: 2020-04-07 22:40:27.380724

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '3c5285bbbb66'
down_revision = '23727a96713b'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('users', sa.Column('created_by_id', sa.Integer(), nullable=True))
    op.add_column('users', sa.Column('invited_by_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'users', 'users', ['created_by_id'], ['id'], ondelete='SET NULL')
    op.create_foreign_key(None, 'users', 'users', ['invited_by_id'], ['id'], ondelete='SET NULL')


def downgrade():
    op.drop_column('users', 'created_by_id')
    op.drop_column('users', 'invited_by_id')



"""max_cards_limit

Revision: 0022
Revision ID: 10cf30e1e791
Revises: d4628b068b5
Create Date: 2015-09-06 14:27:52.473930

"""

# revision identifiers, used by Alembic.
revision = '10cf30e1e791'
down_revision = 'd4628b068b5'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('card_lists', sa.Column('max_cards_limit', sa.Integer(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('card_lists', 'max_cards_limit')
    ### end Alembic commands ###



"""forms_editable_after_submit

Revision: 0183
Revision ID: 3b4dea6ade9d
Revises: 47ecf6e8811b
Create Date: 2020-04-21 09:30:04.775134

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '3b4dea6ade9d'
down_revision = '47ecf6e8811b'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('request_forms', sa.Column('editable_after_submit', sa.Boolean(), nullable=True))
    op.execute("UPDATE request_forms SET editable_after_submit = false")


def downgrade():
    op.drop_column('request_forms', 'editable_after_submit')



"""cardmodel_hide_children_attrs

Revision ID: 0282
Revises: 0281
Create Date: 2025-12-17 13:55:33.711351

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0282'
down_revision = '0281'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('card_models', sa.Column('hide_children_attributes', sa.Boolean()))
    op.execute("UPDATE card_models SET hide_children_attributes = false")


def downgrade():
    op.drop_column('card_models', 'hide_children_attributes')



"""add_push_id

Revision: 0015
Revision ID: 1993ef6216c
Revises: 5a9e03d271b2
Create Date: 2015-07-09 18:00:41.774919

"""

# revision identifiers, used by Alembic.
revision = '1993ef6216c'
down_revision = '5a9e03d271b2'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('cards', sa.Column('push_id', sa.String(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('cards', 'push_id')
    ### end Alembic commands ###



"""card_attributes linked with card_model_attributes

Revision: 0040
Revision ID: 23e5cefde476
Revises: 9f215a58d65
Create Date: 2015-11-30 15:33:07.060545

"""

# revision identifiers, used by Alembic.
revision = '23e5cefde476'
down_revision = '9f215a58d65'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

orgs_table = sa.Table(
    'organizations',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
)

cards_table = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('model_id', sa.Integer),
    sa.Column('parents_ids', postgresql.ARRAY(sa.Integer)),
    sa.Column('default_child_card_model_id', sa.Integer)
)

card_enabled_models_table = sa.Table(
    'card_enabled_models',
    sa.MetaData(),
    sa.Column('model_id', sa.Integer),
    sa.Column('card_id', sa.Integer)
)

card_attributes_table = sa.Table(
    'card_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('type_name', sa.String),
    sa.Column('card_model_attribute_id', sa.Integer)
)

card_models_table = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer)
)


card_model_attributes_table = sa.Table(
    'card_model_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_id', sa.Integer),
    sa.Column('card_model_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('title', sa.String),
    sa.Column('type_name', sa.String)
)

def upgrade():
    op.add_column('card_attributes', sa.Column('card_model_attribute_id', sa.Integer(), nullable=True))
    op.create_foreign_key('card_attribute_card_model_attribute_id', 'card_attributes', 'card_model_attributes', ['card_model_attribute_id'], ['id'], ondelete='CASCADE')

    conn = op.get_bind()
    for org in conn.execute(orgs_table.select()):
        model_attributes = {}
        for model in conn.execute(card_models_table.select().where(card_models_table.c.owner_id == org.id)):
            for model_attr in conn.execute(card_model_attributes_table.select().where(card_model_attributes_table.c.card_model_id == model.id)):
                key = model_attr.type_name + '::' + model_attr.name + '!!' + str(model_attr.card_model_id)
                if key not in model_attributes:
                    model_attributes[key] = model_attr.id

        def updateCardAttr(key, attr):
            keys = []
            keys.append(key)
            key2 = ''.join(key).replace('_', ' ')
            if key2 not in keys:
                keys.append(key2)
            key2 = ''.join(key).replace(' ', '_')
            if key2 not in keys:
                keys.append(key2)
            for k in keys:
                if k in model_attributes:
                    conn.execute(card_attributes_table.update().where(card_attributes_table.c.id == attr.id).values(card_model_attribute_id=model_attributes[k]))
                    return True
            return False

        models = {}
        enabled = {}
        org_models = []
        for card in conn.execute(cards_table.select().where(cards_table.c.owner_id == org.id)):
            top_level_card = card
            for attr in conn.execute(card_attributes_table.select().where(card_attributes_table.c.card_id == card.id)):
                if 'github' in attr.type_name or attr.name == 'github_branch' or 'gitlab_' in attr.name:
                    continue
                key = attr.type_name + '::' + attr.name + '!!' + str(card.model_id)
                if updateCardAttr(key, attr):
                    continue
                if len(card.parents_ids):
                    top_level_card = conn.execute(cards_table.select().where(cards_table.c.id == card.parents_ids[0])).first()
                if top_level_card.id not in models:
                    models[top_level_card.id] = []
                    models[top_level_card.id].append(top_level_card.model_id)
                    models[top_level_card.id].append(top_level_card.default_child_card_model_id)

                found = False
                for model_id in models[top_level_card.id]:
                    if updateCardAttr(attr.type_name + '::' + attr.name + '!!' + str(model_id), attr):
                        found = True
                        break
                if found:
                    continue
                if top_level_card.id not in enabled:
                    enabled[top_level_card.id] = []
                    for model in conn.execute(card_enabled_models_table.select().where(card_enabled_models_table.c.card_id == top_level_card.id)):
                        enabled[top_level_card.id].append(model.model_id)
                for model_id in enabled[top_level_card.id]:
                    if updateCardAttr(attr.type_name + '::' + attr.name + '!!' + str(model_id), attr):
                        found = True
                        break
                if found:
                    continue
                if not len(org_models):
                    for model in conn.execute(card_models_table.select().where(card_models_table.c.owner_id == org.id)):
                        org_models.append(model.id)
                for model_id in org_models:
                    if updateCardAttr(attr.type_name + '::' + attr.name + '!!' + str(model_id), attr):
                        found = True
                        break
                if found:
                    continue
                key = attr.type_name + '::' + attr.name + '!!extra:' + str(card.id)
                for extra_attr in conn.execute(card_model_attributes_table.select().where(sa.and_(card_model_attributes_table.c.card_id == card.id, card_model_attributes_table.c.card_model_id == None))):
                    if extra_attr.name == attr.name and extra_attr.type_name == attr.type_name:
                        model_attributes[key] = extra_attr.id
                        updateCardAttr(key, attr)
                        found = True
                        break
                if found:
                    continue
                res = conn.execute(card_model_attributes_table.insert().values(card_id=card.id, name=attr.name, title=attr.name, type_name=attr.type_name))
                model_attributes[key] = res.inserted_primary_key[0]
                updateCardAttr(key, attr)


def downgrade():
    op.drop_constraint('card_attribute_card_model_attribute_id', 'card_attributes', type_='foreignkey')
    op.drop_column('card_attributes', 'card_model_attribute_id')



"""google_integration

Revision: 0127
Revision ID: 4e8938a1c66
Revises: 431110172682
Create Date: 2018-04-24 01:08:39.007577

"""

# revision identifiers, used by Alembic.
revision = '4e8938a1c66'
down_revision = '431110172682'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('google_token', sa.String(), nullable=True))
    op.add_column('users', sa.Column('google_refresh_token', sa.String(), nullable=True))
    op.add_column('users', sa.Column('google_id', sa.String(), nullable=True))
    op.add_column('users', sa.Column('google_name', sa.String(), nullable=True))
    op.add_column('users', sa.Column('google_email', sa.String(), nullable=True))
    op.create_index(op.f('ix_users_google_id'), 'users', ['google_id'], unique=False)


def downgrade():
    op.drop_column('users', 'google_token')
    op.drop_column('users', 'google_refresh_token')
    op.drop_column('users', 'google_id')
    op.drop_column('users', 'google_name')
    op.drop_column('users', 'google_email')



"""kql_flat

Revision: 0160
Revision ID: 1ef3e95bf70d
Revises: 2f1187b05f65
Create Date: 2019-02-19 22:34:16.800414

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '1ef3e95bf70d'
down_revision = '2f1187b05f65'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data timestamp[]) RETURNS timestamp
        AS
        $BODY$
            SELECT data[1];
        $BODY$
        LANGUAGE sql;
    """)


    op.execute("""
        CREATE OR REPLACE FUNCTION kql_flat(data timestamp) RETURNS timestamp
        AS
        $BODY$
            SELECT data;
        $BODY$
        LANGUAGE sql;
    """)


def downgrade():
    op.execute("DROP FUNCTION kql_flat(timestamp[])")
    op.execute("DROP FUNCTION kql_flat(timestamp)")



"""add_share_object_options

Revision ID: 0252
Revises: 0251
Create Date: 2023-05-11 16:46:03.926683

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0252'
down_revision = '0251'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('share_objects', sa.Column('options', postgresql.JSONB()))


def downgrade():
    op.delete_column('share_objects', 'options')



"""convert_group_type_scope

Revision: 0132
Revision ID: 4c4163218424
Revises: 3feaab6b3df
Create Date: 2018-04-17 11:21:29.082214

"""

# revision identifiers, used by Alembic.
revision = '4c4163218424'
down_revision = '3feaab6b3df'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa

card_group_types = sa.Table(
    'card_group_types',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('scope', sa.String),
    sa.Column('cache_id', sa.String)
)

card_groups = sa.Table(
    'card_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('group_type_id', sa.Integer),
    sa.Column('parent_id', sa.Integer),
    sa.Column('title', sa.String),
    sa.Column('cache_id', sa.String)
)

card_groups_cards = sa.Table(
    'card_groups_cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_id', sa.Integer),
    sa.Column('group_id', sa.Integer)
)

def upgrade():
    conn = op.get_bind()

    # select group types which have groups with only one parent_id or no groups at all
    context_to_convert_directly_query = """
        select cgt.id from card_group_types cgt
        left join (select group_type_id, count(distinct parent_id) c from card_groups group by group_type_id) cg on cg.group_type_id = cgt.id
        where cgt.scope = 'card' and (cg.c <= 1 or cg.c is null)
    """

    updates = [{'gtid': r[0]} for r in conn.execute(context_to_convert_directly_query)]
    if updates:
        conn.execute(card_group_types.update().where(card_group_types.c.id==sa.bindparam('gtid')).values(
            scope='project', cache_id=sa.func.uuid_generate_v4()), updates)


    groups_attr_to_convert_directly_query = """
        select cgt.id from card_group_types cgt
        left join (select group_type_id, count(distinct parent_id) c from card_groups group by group_type_id) cg on cg.group_type_id = cgt.id
        where cgt.scope = 'project' and cgt.attribute_id is not null and (cg.c <= 1 or cg.c is null)
    """

    updates = [{'gtid': r[0]} for r in conn.execute(groups_attr_to_convert_directly_query)]
    if updates:
        conn.execute(card_group_types.update().where(card_group_types.c.id==sa.bindparam('gtid')).values(
            scope='org', cache_id=sa.func.uuid_generate_v4()), updates)
        conn.execute(card_groups.update().where(card_groups.c.group_type_id==sa.bindparam('gtid')).values(
            parent_id=None, cache_id=sa.func.uuid_generate_v4()), updates)


    groups_attr_to_process_query = """
        select cgt.id from card_group_types cgt
        left join (select group_type_id, count(distinct parent_id) c from card_groups group by group_type_id) cg on cg.group_type_id = cgt.id
        where cgt.scope = 'project' and cgt.attribute_id is not null and cg.c > 1
    """

    scope_updates = []
    parent_updates = []
    remap_updates = []
    for r in conn.execute(groups_attr_to_process_query):
        scope_updates.append({'gtid': r[0]})
        group_by_titles = {}
        for g in conn.execute(card_groups.select().where(card_groups.c.group_type_id==r[0])):
            group_by_titles.setdefault(g.title, []).append(g)
        for title, groups in group_by_titles.items():
            parent_updates.append({'gid': groups[0].id})
            if len(groups) == 1:
                continue
            for g in groups[1:]:
                remap_updates.append({'from_id': g.id, 'to_id': groups[0].id})

    if scope_updates:
        conn.execute(card_group_types.update().where(card_group_types.c.id==sa.bindparam('gtid')).values(
            scope='org', cache_id=sa.func.uuid_generate_v4()), scope_updates)
    if remap_updates:
        cgc1 = card_groups_cards.alias()
        cgc2 = card_groups_cards.alias()
        exclude_cards = sa.select([cgc1.c.card_id]).select_from(
            cgc1.join(cgc2, cgc2.c.card_id==cgc1.c.card_id)
        ).where(
            sa.and_(cgc1.c.group_id==sa.bindparam('from_id'), cgc2.c.group_id==sa.bindparam('to_id'))
        )

        conn.execute(card_groups_cards.update().where(sa.and_(
            card_groups_cards.c.group_id==sa.bindparam('from_id'),
            ~card_groups_cards.c.card_id.in_(exclude_cards)
        )).values(group_id=sa.bindparam('to_id')), remap_updates)

        conn.execute(card_groups.delete().where(card_groups.c.id==sa.bindparam('from_id')), remap_updates)
    if parent_updates:
        conn.execute(card_groups.update().where(card_groups.c.id==sa.bindparam('gid')).values(
            parent_id=None, cache_id=sa.func.uuid_generate_v4()), parent_updates)

    op.execute("""
        WITH positions as (
            SELECT id, row_number() OVER (partition by group_type_id order by position) AS p
            FROM card_groups WHERE NOT is_archived
        ) UPDATE card_groups SET position = positions.p - 1 FROM positions, card_group_types cgt
        WHERE cgt.id = card_groups.group_type_id AND cgt.attribute_id IS NOT NULL AND positions.id = card_groups.id
    """)

def downgrade():
    pass



"""cards_image_cover_id

Revision: 0050
Revision ID: 44d8f57c0e76
Revises: 1336f79b826c
Create Date: 2016-03-03 11:47:53.360437

"""

# revision identifiers, used by Alembic.
revision = '44d8f57c0e76'
down_revision = '1336f79b826c'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.add_column('cards', sa.Column('image_cover_id', sa.Integer(), nullable=True))
    op.create_index(op.f('ix_cards_image_cover_id'), 'cards', ['image_cover_id'], unique=False)
    op.create_foreign_key(None, 'cards', 'card_files', ['image_cover_id'], ['id'], ondelete='SET NULL')


def downgrade():
    op.drop_index(op.f('ix_cards_image_cover_id'), table_name='cards')
    op.drop_column('cards', 'image_cover_id')



"""project_slug

Revision: 0074
Revision ID: 205096351a13
Revises: 1b3bcd25ba75
Create Date: 2016-10-04 12:10:08.029176

"""

# revision identifiers, used by Alembic.
revision = '205096351a13'
down_revision = '1b3bcd25ba75'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('projects', sa.Column('slug', sa.String(), nullable=True))
    op.create_index(op.f('ix_projects_slug'), 'projects', ['slug'], unique=True)
    op.execute("""UPDATE projects SET slug = (SELECT slug FROM cards WHERE id = projects.top_level_card_id)""")


def downgrade():
    op.drop_index(op.f('ix_projects_slug'), table_name='projects')
    op.drop_column('projects', 'slug')



"""group_state_alias

Revision: 0068
Revision ID: 491bd448c7c8
Revises: 31f84a385d20
Create Date: 2016-08-10 16:09:04.900187

"""

# revision identifiers, used by Alembic.
revision = '491bd448c7c8'
down_revision = '31f84a385d20'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("UPDATE cards SET state_alias_id = NULL")

    op.drop_index(op.f('ix_cards_state_alias_id'), table_name='cards')
    op.drop_constraint('cards_state_alias_id_fkey', 'cards')
    op.create_foreign_key(None, 'cards', 'card_groups', ['state_alias_id'], ['id'])
    op.create_index(op.f('ix_cards_state_alias_id'), 'cards', ['state_alias_id'], unique=False)
    
    op.drop_index(op.f('ix_card_state_aliases_label'), table_name='card_state_aliases')
    op.drop_index(op.f('ix_card_state_aliases_card_id'), table_name='card_state_aliases')
    op.drop_table('card_state_aliases')


def downgrade():
    pass



"""convert_choice_attr_to_groups

Revision: 0133
Revision ID: 4cee323fc652
Revises: 4c4163218424
Create Date: 2018-04-17 12:11:17.101847

"""

# revision identifiers, used by Alembic.
revision = '4cee323fc652'
down_revision = '4c4163218424'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import select
from sqlalchemy.dialects import postgresql
import datetime


attributes = sa.Table(
    'attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('group_id', sa.Integer),
    sa.Column('type_name', sa.String),
    sa.Column('name', sa.String),
    sa.Column('is_from_integration', sa.String),
    sa.Column('options', sa.PickleType)
)

attribute_groups = sa.Table(
    'attribute_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer)
)

cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('cache_id', sa.String)
)

card_attributes = sa.Table(
    'card_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('attribute_id', sa.Integer),
    sa.Column('card_id', sa.Integer),
    sa.Column('value', postgresql.JSONB)
)

card_group_types = sa.Table(
    'card_group_types',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('created_at', sa.DateTime),
    sa.Column('owner_id', sa.Integer),
    sa.Column('attribute_id', sa.Integer),
    sa.Column('scope', sa.String),
    sa.Column('one_group_per_card', sa.Boolean),
    sa.Column('auto_random_color', sa.Boolean),
    sa.Column('display_on_card', sa.String),
    sa.Column('is_project_specific', sa.Boolean),
    sa.Column('cache_id', sa.String)
)

card_groups = sa.Table(
    'card_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('created_at', sa.DateTime),
    sa.Column('group_type_id', sa.Integer),
    sa.Column('parent_id', sa.Integer),
    sa.Column('title', sa.String),
    sa.Column('position', sa.Integer),
    sa.Column('is_archived', sa.Boolean),
    sa.Column('show_flow_indicator', sa.Boolean),
    sa.Column('always_show_description', sa.Boolean),
    sa.Column('disallow_direct_card_creation', sa.Boolean),
    sa.Column('cache_id', sa.String)
)

card_groups_cards = sa.Table(
    'card_groups_cards',
    sa.MetaData(),
    sa.Column('card_id', sa.Integer),
    sa.Column('group_id', sa.Integer),
    sa.Column('position', sa.Integer)
)

card_models = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('attribute_group_id', sa.Integer),
    sa.Column('cache_id', sa.String)
)


def upgrade():
    conn = op.get_bind()

    for attr in conn.execute(attributes.select().where(sa.and_(attributes.c.type_name == 'choice', attributes.c.is_from_integration == None))):
        owner_id = conn.execute(select([attribute_groups.c.owner_id]).where(attribute_groups.c.id==attr.group_id)).scalar()
        multi = attr.options and attr.options.get('multi')
        r = conn.execute(card_group_types.insert().values(
            created_at=datetime.datetime.utcnow(),
            owner_id=owner_id,
            attribute_id=attr.id,
            scope='org',
            one_group_per_card=not multi,
            auto_random_color=False,
            display_on_card='label',
            is_project_specific=False,
            cache_id=sa.func.uuid_generate_v4()
        ))
        gt_id = r.inserted_primary_key[0]

        conn.execute(attributes.update().where(attributes.c.id==attr.id).values(
            type_name='group_type', options={'one_group_per_card': not multi, 'auto_random_color': False}))

        if attr.options and attr.options.get('items'):
            for i, title in enumerate(attr.options['items']):
                r = conn.execute(card_groups.insert().values(
                    created_at=datetime.datetime.utcnow(),
                    group_type_id=gt_id,
                    parent_id=None,
                    title=title,
                    position=i,
                    is_archived=False,
                    show_flow_indicator=False,
                    always_show_description=False,
                    disallow_direct_card_creation=False,
                    cache_id=sa.func.uuid_generate_v4()
                ))
                gid = r.inserted_primary_key[0]
                q = select([card_attributes.c.card_id], distinct=True).where(
                        sa.and_(card_attributes.c.attribute_id==attr.id, card_attributes.c.value.op('?')(title)))
                for j, r in enumerate(conn.execute(q)):
                    conn.execute(card_groups_cards.insert().values(group_id=gid, card_id=r[0], position=j))

        conn.execute(cards.update().where(sa.and_(card_attributes.c.card_id == cards.c.id, card_attributes.c.attribute_id==attr.id))\
            .values(cache_id=sa.func.uuid_generate_v4()))
        conn.execute(card_attributes.delete().where(card_attributes.c.attribute_id==attr.id))

    conn.execute(card_models.update().where(sa.and_(attributes.c.group_id == card_models.c.attribute_group_id, attributes.c.type_name == 'group_type'))\
        .values(cache_id=sa.func.uuid_generate_v4()))


def downgrade():
    pass



"""org_subscription_required

Revision: 0039
Revision ID: 9f215a58d65
Revises: 448178393554
Create Date: 2015-11-29 11:27:51.279717

"""

# revision identifiers, used by Alembic.
revision = '9f215a58d65'
down_revision = '448178393554'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.add_column('organizations', sa.Column('subscription_required', sa.Boolean(), nullable=True))
    op.add_column('organizations', sa.Column('payment_method', sa.String(), nullable=True))
    op.execute("UPDATE organizations SET subscription_required = true, payment_method = 'stripe'")


def downgrade():
    op.drop_column('organizations', 'subscription_required')
    op.drop_column('organizations', 'payment_method')


"""user_sub_required

Revision: 0043
Revision ID: 459ea8446830
Revises: da5397402d5
Create Date: 2015-12-16 07:56:15.797150

"""

# revision identifiers, used by Alembic.
revision = '459ea8446830'
down_revision = 'da5397402d5'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('subscription_required', sa.Boolean(), nullable=True))
    op.execute("UPDATE users SET subscription_required = true")


def downgrade():
    op.drop_column('users', 'subscription_required')



"""card_started_at

Revision: 0086
Revision ID: 37726611a9ea
Revises: 2d68c5dd350e
Create Date: 2017-03-03 11:25:26.783510

"""

# revision identifiers, used by Alembic.
revision = '37726611a9ea'
down_revision = '2d68c5dd350e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('cards', sa.Column('started_at', sa.DateTime(), nullable=True))


def downgrade():
    op.drop_column('cards', 'started_at')



"""cleanup_card_attributes_json_nulls

Revision ID: 0251
Revises: 0250
Create Date: 2023-05-05 21:20:02.408801

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0251'
down_revision = '0250'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        DELETE FROM card_attributes
        WHERE (value IS NULL OR value = 'null'::jsonb) AND unindexed_value IS NULL
    """)


def downgrade():
    pass



"""fix_project_members_role_id_after_move

Revision ID: 0265
Revises: 0264
Create Date: 2025-02-05 13:16:17.623883

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0265'
down_revision = '0264'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE project_members pm
        SET role_id = (
            SELECT r2.id FROM roles r2 WHERE (r.builtin = r2.builtin OR r.name = r2.name)
            AND r2.owner_id = p.owner_id
        )
        FROM projects p, roles r
        WHERE pm.project_id = p.id AND r.id = pm.role_id AND p.owner_id != r.owner_id
    """)


def downgrade():
    pass


"""stripe

Revision: 0026
Revision ID: 4c29125461b8
Revises: 568bf972b726
Create Date: 2015-09-21 12:07:24.890658

"""

# revision identifiers, used by Alembic.
revision = '4c29125461b8'
down_revision = '568bf972b726'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.create_table('invoices',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('ref', sa.String(), nullable=True),
        sa.Column('currency', sa.String(length=3), nullable=True),
        sa.Column('subtotal', sa.Float(), nullable=True),
        sa.Column('total', sa.Float(), nullable=True),
        sa.Column('tax_rate', sa.Float(), nullable=True),
        sa.Column('tax_amount', sa.Float(), nullable=True),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('email', sa.String(), nullable=True),
        sa.Column('address_line1', sa.String(), nullable=True),
        sa.Column('address_line2', sa.String(), nullable=True),
        sa.Column('address_city', sa.String(), nullable=True),
        sa.Column('address_state', sa.String(), nullable=True),
        sa.Column('address_zip', sa.String(), nullable=True),
        sa.Column('address_country', sa.String(), nullable=True),
        sa.Column('country', sa.String(length=2), nullable=True),
        sa.Column('customer_special_mention', sa.String(), nullable=True),
        sa.Column('issued_at', sa.DateTime(), nullable=True),
        sa.Column('charge_id', sa.String(), nullable=True),
        sa.Column('external_id', sa.String(), nullable=True),
        sa.Column('customer_id', sa.Integer(), nullable=True),
        sa.Column('is_eu_country', sa.Boolean(), nullable=True),
        sa.Column('eu_vat_number', sa.String(), nullable=True),
        sa.Column('eu_exchange_rate', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['customer_id'], ['organizations.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_invoices_customer_id'), 'invoices', ['customer_id'], unique=False)

    op.create_table('invoice_items',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('invoice_id', sa.Integer(), nullable=True),
        sa.Column('amount', sa.Float(), nullable=True),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('quantity', sa.Integer(), nullable=True),
        sa.Column('subtotal', sa.Float(), nullable=True),
        sa.Column('currency', sa.String(length=3), nullable=True),
        sa.Column('external_id', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['invoice_id'], ['invoices.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_invoice_items_invoice_id'), 'invoice_items', ['invoice_id'], unique=False)

    op.add_column('organizations', sa.Column('billing_address_city', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_address_country', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_address_line1', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_address_line2', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_address_state', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_address_zip', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_brand', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_country', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_exp_month', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_exp_year', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_ip_address', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_last4', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('billing_name', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('email', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('eu_vat_country', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('eu_vat_number', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('eu_vat_rate', sa.Float(), nullable=True))
    op.add_column('organizations', sa.Column('has_stripe_source', sa.Boolean(), nullable=True))
    op.add_column('organizations', sa.Column('plan_last_charge_amount', sa.Float(), nullable=True))
    op.add_column('organizations', sa.Column('plan_last_charge_successful', sa.Boolean(), nullable=True))
    op.add_column('organizations', sa.Column('plan_last_charged_at', sa.DateTime(), nullable=True))
    op.add_column('organizations', sa.Column('plan_name', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('plan_next_charge_at', sa.DateTime(), nullable=True))
    op.add_column('organizations', sa.Column('plan_status', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('stripe_customer_id', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('stripe_subscription_id', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('telephone', sa.String(), nullable=True))
    op.add_column('organizations', sa.Column('company', sa.String(), nullable=True))
    op.create_index(op.f('ix_organizations_trial_ends_at'), 'organizations', ['trial_ends_at'], unique=False)
    op.create_index(op.f('ix_organizations_has_stripe_source'), 'organizations', ['has_stripe_source'], unique=False)
    op.create_index(op.f('ix_organizations_plan_next_charge_at'), 'organizations', ['plan_next_charge_at'], unique=False)
    op.create_index(op.f('ix_organizations_plan_status'), 'organizations', ['plan_status'], unique=False)
    op.create_index(op.f('ix_organizations_stripe_customer_id'), 'organizations', ['stripe_customer_id'], unique=False)
    op.create_index(op.f('ix_organizations_stripe_subscription_id'), 'organizations', ['stripe_subscription_id'], unique=False)
    op.execute('UPDATE organizations SET plan_status = status')
    op.drop_column('organizations', 'status')


def downgrade():
    op.add_column('organizations', sa.Column('status', sa.VARCHAR(), autoincrement=False, nullable=True))
    op.execute('UPDATE organizations SET status = plan_status')
    op.drop_index(op.f('ix_organizations_stripe_subscription_id'), table_name='organizations')
    op.drop_index(op.f('ix_organizations_stripe_customer_id'), table_name='organizations')
    op.drop_index(op.f('ix_organizations_plan_status'), table_name='organizations')
    op.drop_index(op.f('ix_organizations_plan_next_charge_at'), table_name='organizations')
    op.drop_index(op.f('ix_organizations_has_stripe_source'), table_name='organizations')
    op.drop_index(op.f('ix_organizations_trial_ends_at'), table_name='organizations')
    op.drop_column('organizations', 'company')
    op.drop_column('organizations', 'telephone')
    op.drop_column('organizations', 'stripe_subscription_id')
    op.drop_column('organizations', 'stripe_customer_id')
    op.drop_column('organizations', 'plan_status')
    op.drop_column('organizations', 'plan_next_charge_at')
    op.drop_column('organizations', 'plan_name')
    op.drop_column('organizations', 'plan_last_charged_at')
    op.drop_column('organizations', 'plan_last_charge_successful')
    op.drop_column('organizations', 'plan_last_charge_amount')
    op.drop_column('organizations', 'has_stripe_source')
    op.drop_column('organizations', 'eu_vat_rate')
    op.drop_column('organizations', 'eu_vat_number')
    op.drop_column('organizations', 'eu_vat_country')
    op.drop_column('organizations', 'email')
    op.drop_column('organizations', 'billing_name')
    op.drop_column('organizations', 'billing_last4')
    op.drop_column('organizations', 'billing_ip_address')
    op.drop_column('organizations', 'billing_exp_year')
    op.drop_column('organizations', 'billing_exp_month')
    op.drop_column('organizations', 'billing_country')
    op.drop_column('organizations', 'billing_brand')
    op.drop_column('organizations', 'billing_address_zip')
    op.drop_column('organizations', 'billing_address_state')
    op.drop_column('organizations', 'billing_address_line2')
    op.drop_column('organizations', 'billing_address_line1')
    op.drop_column('organizations', 'billing_address_country')
    op.drop_column('organizations', 'billing_address_city')

    op.drop_index(op.f('ix_invoice_items_invoice_id'), table_name='invoice_items')
    op.drop_table('invoice_items')
    op.drop_index(op.f('ix_invoices_customer_id'), table_name='invoices')
    op.drop_table('invoices')



"""system_message

Revision: 0078
Revision ID: 4e9a229614fb
Revises: f6be47c2049
Create Date: 2016-12-06 20:28:55.915370

"""

# revision identifiers, used by Alembic.
revision = '4e9a229614fb'
down_revision = 'f6be47c2049'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('system_message', sa.Text(), nullable=True))


def downgrade():
    op.drop_column('users', 'system_message')



"""default_views

Revision: 0053
Revision ID: 78a2be497a8
Revises: 7790b50d0be
Create Date: 2016-03-20 10:27:35.183434

"""

# revision identifiers, used by Alembic.
revision = '78a2be497a8'
down_revision = '7790b50d0be'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa

def upgrade():
    op.execute("""
        INSERT INTO views (card_id, title, icon, group_type_id, mode, mode_options)
        SELECT
            c.id, 'Default', 'home', c.default_card_group_type_id, 'kanban', '{"showUngroupedCards": true}'::jsonb
        FROM
            cards c
        WHERE
            c.parent_id is null
    """)
    op.execute("""
        INSERT INTO views (card_id, title, icon, group_type_id, filter, mode, mode_options)
        SELECT
            c.id, 'My cards', 'check-square-o', c.default_card_group_type_id, '@me', 'table', '{"showUngroupedCards": true}'::jsonb
        FROM
            cards c
        WHERE
            c.parent_id is null
    """)
    op.execute("""
        INSERT INTO views (card_id, title, icon, group_type_id, filter, mode, mode_options)
        SELECT
            c.id, 'Due this week', 'calendar-check-o', c.default_card_group_type_id, 'due:week', 'kanban', '{"showUngroupedCards": true}'::jsonb
        FROM
            cards c
        WHERE
            c.parent_id is null
    """)


def downgrade():
    pass



"""disallow_card_creation_in_list

Revision: 0025
Revision ID: 568bf972b726
Revises: 188181cb15ef
Create Date: 2015-09-15 12:39:18.389728

"""

# revision identifiers, used by Alembic.
revision = '568bf972b726'
down_revision = '188181cb15ef'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.add_column('card_lists', sa.Column('disallow_direct_card_creation', sa.Boolean(), nullable=True))
    op.execute('UPDATE card_lists SET disallow_direct_card_creation = false')


def downgrade():
    op.drop_column('card_lists', 'disallow_direct_card_creation')


"""portfolio

Revision ID: 0258
Revises: 0257
Create Date: 2023-03-22 15:51:49.836478

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0258'
down_revision = '0257'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('projects', sa.Column('is_portfolio', sa.Boolean(), nullable=False, server_default="false"))
    op.create_index(op.f('ix_projects_is_portfolio'), 'projects', ['is_portfolio'])

    op.create_table('portfolio_projects',
        sa.Column('portfolio_id', sa.Integer()),
        sa.Column('project_id', sa.Integer()),
        sa.ForeignKeyConstraint(['portfolio_id'], ['projects.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('portfolio_id', 'project_id')
    )
    op.create_index(op.f('ix_portfolio_projects_portfolio_id'), 'portfolio_projects', ['portfolio_id'])
    op.create_index(op.f('ix_portfolio_projects_portfolio_id_project_id'), 'portfolio_projects', ['portfolio_id', 'project_id'])


def downgrade():
    op.drop_table('portfolio_projects')
    op.drop_column('projects', 'is_portfolio')



"""signup_cpc

Revision: 0146
Revision ID: 51e932e2e7f7
Revises: 2e10b78f004e
Create Date: 2018-10-25 21:44:07.672445

"""

# revision identifiers, used by Alembic.
revision = '51e932e2e7f7'
down_revision = '2e10b78f004e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.drop_column('users', 'signup_utma')
    op.drop_column('users', 'signup_utmz')
    op.add_column('users', sa.Column('signup_country', sa.String(), nullable=True))
    op.add_column('users', sa.Column('signup_cpc_source', sa.String(), nullable=True))
    op.add_column('users', sa.Column('signup_cpc_campaign', sa.String(), nullable=True))
    op.add_column('users', sa.Column('signup_cpc_ad', sa.String(), nullable=True))
    op.add_column('users', sa.Column('signup_cpc_keyword', sa.String(), nullable=True))
    op.add_column('users', sa.Column('signup_cpc_sessionid', sa.String(), nullable=True))
    op.create_index(op.f('ix_users_signup_country'), 'users', ['signup_country'], unique=False)
    op.create_index(op.f('ix_users_signup_cpc_source'), 'users', ['signup_cpc_source'], unique=False)


def downgrade():
    op.drop_column('users', 'signup_country')
    op.drop_column('users', 'signup_cpc_source')
    op.drop_column('users', 'signup_cpc_campaign')
    op.drop_column('users', 'signup_cpc_ad')
    op.drop_column('users', 'signup_cpc_keyword')
    op.drop_column('users', 'signup_cpc_sessionid')
    op.add_column('users', sa.Column('signup_utma', postgresql.JSONB(), nullable=True))
    op.add_column('users', sa.Column('signup_utmz', postgresql.JSONB(), nullable=True))



"""user_onboarded

Revision: 0065
Revision ID: 2daf3bc83d75
Revises: 24191069ca2a
Create Date: 2016-07-15 12:03:47.279709

"""

# revision identifiers, used by Alembic.
revision = '2daf3bc83d75'
down_revision = '24191069ca2a'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('onboarded', sa.Boolean(), nullable=True))
    op.execute("UPDATE users SET onboarded = true")


def downgrade():
    op.drop_column('users', 'onboarded')



"""fix_children_default_model

Revision: 0109
Revision ID: 50312a20c5ea
Revises: 7583b4a2428
Create Date: 2018-01-09 15:20:37.912624

"""

# revision identifiers, used by Alembic.
revision = '50312a20c5ea'
down_revision = '7583b4a2428'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa

projects = sa.Table(
    'projects',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True)
)

enabled_models = sa.Table(
    'project_enabled_models',
    sa.MetaData(),
    sa.Column('project_id', sa.Integer),
    sa.Column('model_id', sa.Integer)
)

models = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('name', sa.String()),
    sa.Column('children_default_model_id', sa.Integer),
    sa.Column('is_project_specific', sa.Boolean)
)

def upgrade():
    conn = op.get_bind()

    projects2 = sa.alias(projects)
    enabled_models2 = sa.alias(enabled_models)
    models2 = sa.alias(models)

    projects3 = sa.alias(projects)
    enabled_models3 = sa.alias(enabled_models)
    models3 = sa.alias(models)

    q = sa.select([projects.c.id, models.c.id, models2.c.id, models3.c.id]).where(sa.and_(models.c.children_default_model_id == models2.c.id,
                                      models.c.id == enabled_models.c.model_id,
                                      enabled_models.c.project_id == projects.c.id,
                                      models2.c.is_project_specific == True,
                                      models2.c.id == enabled_models2.c.model_id,
                                      enabled_models2.c.project_id == projects2.c.id,
                                      projects.c.id != projects2.c.id,
                                      models3.c.id == enabled_models3.c.model_id,
                                      enabled_models3.c.project_id == projects3.c.id,
                                      projects3.c.id == projects.c.id,
                                      models3.c.name == models2.c.name))

    model_ids = []
    for model in conn.execute(q):
        op.execute("UPDATE cards SET model_id = {0} WHERE project_id = {1} AND model_id = {2}".format(model[3], model[0], model[2]))
        model_ids.append({'model_id':model[1], 'children_model_id':model[3]})

    if model_ids:
        conn.execute(models.update().where(models.c.id == sa.bindparam('model_id')).values(
            children_default_model_id=sa.bindparam('children_model_id')), model_ids)

def downgrade():
    pass



"""disable_ai_assistant

Revision ID: 0276
Revises: 0275
Create Date: 2025-09-05

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0276'
down_revision = '0275'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('organizations', sa.Column('ai_assistant_enabled', sa.Boolean(), nullable=False, server_default="false"))


def downgrade():
    op.drop_column('organizations', 'ai_assistant_enabled')



"""template_icon_and_oneliner

Revision: 0101
Revision ID: 38487891c2c0
Revises: 3b0d34ece0d5
Create Date: 2017-07-13 18:59:27.544395

"""

# revision identifiers, used by Alembic.
revision = '38487891c2c0'
down_revision = '3b0d34ece0d5'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa

def upgrade():
    op.add_column('projects', sa.Column('icon', sa.String(), nullable=True))
    op.add_column('projects', sa.Column('oneliner', sa.Text(), nullable=True))

def downgrade():
    op.drop_column('projects', 'oneliner')
    op.drop_column('projects', 'icon')


"""group_type_attributes_display_on_card_label

Revision: 0152
Revision ID: 4fd1516bbd51
Revises: 3124afe3927f
Create Date: 2018-12-04 15:13:45.509776

"""

# revision identifiers, used by Alembic.
revision = '4fd1516bbd51'
down_revision = '3124afe3927f'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        UPDATE card_group_types SET display_on_card='label' FROM attributes
        WHERE card_group_types.attribute_id=attributes.id AND attributes.show_on_tile=true AND card_group_types.display_on_card IS NULL
    """)


def downgrade():
    pass



"""automation_saved_state_triggered_by

Revision ID: 0254
Revises: 0253
Create Date: 2023-06-26 10:40:57.233153

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0254'
down_revision = '0253'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('automation_states', sa.Column('triggered_by_user_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'automation_states', 'users', ['triggered_by_user_id'], ['id'], ondelete='SET NULL')


def downgrade():
    op.drop_column('automation_states', 'triggered_by_user_id')



"""user_license_through_org

Revision ID: 0283
Revises: 0282
Create Date: 2025-12-17 13:55:33.711351

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0283'
down_revision = '0282'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('users', sa.Column('licensed_through_org', sa.Boolean()))
    op.execute("UPDATE users SET licensed_through_org = false")


def downgrade():
    pass



"""disabled_user

Revision: 0059
Revision ID: 4cd3011ada05
Revises: 3bc28e6d9f90
Create Date: 2016-05-20 10:56:23.169068

"""

# revision identifiers, used by Alembic.
revision = '4cd3011ada05'
down_revision = '3bc28e6d9f90'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('is_disabled', sa.Boolean(), nullable=True))
    op.add_column('users', sa.Column('disabled_at', sa.DateTime(), nullable=True))
    op.add_column('users', sa.Column('last_activity_at', sa.DateTime(), nullable=True))
    op.execute("UPDATE users SET is_disabled=false, last_activity_at = last_login_at")
    op.alter_column('users', 'is_disabled', nullable=False)


def downgrade():
    op.drop_column('users', 'last_activity_at')
    op.drop_column('users', 'is_disabled')
    op.drop_column('users', 'disabled_at')


"""billing_activation_date

Revision: 0242
Revision ID: 3769937df895
Revises: 6372d988531f
Create Date: 2022-07-20 16:46:29.658713

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '3769937df895'
down_revision = '6372d988531f'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('billing_accounts', sa.Column('activation_date', sa.DateTime(), nullable=True))


def downgrade():
    op.drop_column('billing_accounts', 'activation_date')



"""user_trial_ends_at

Revision: 0044
Revision ID: 21e56a0093b
Revises: 459ea8446830
Create Date: 2015-12-20 11:06:13.745404

"""

# revision identifiers, used by Alembic.
revision = '21e56a0093b'
down_revision = '459ea8446830'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('trial_ends_at', sa.DateTime(), nullable=True))
    op.execute("UPDATE users SET trial_ends_at = (SELECT trial_ends_at FROM organizations WHERE created_by_id = users.id AND is_user_org)")


def downgrade():
    op.drop_column('users', 'trial_ends_at')



"""unique_user_org

Revision: 0182
Revision ID: 47ecf6e8811b
Revises: 27a56466f9a7
Create Date: 2020-05-07 15:36:26.299226

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '47ecf6e8811b'
down_revision = '27a56466f9a7'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        CREATE UNIQUE INDEX user_org_id
        ON organizations (created_by_id, is_user_org)
        WHERE name IS NULL AND is_user_org IS TRUE;
    """)


def downgrade():
    op.execute("""DROP INDEX IF EXISTS user_org_id""")



"""added is_admin

Revision: 0007
Revision ID: 4ce0659a41fd
Revises: 43064fa5bc36
Create Date: 2015-06-10 19:35:59.527805

"""

# revision identifiers, used by Alembic.
revision = '4ce0659a41fd'
down_revision = '43064fa5bc36'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('users', sa.Column('is_admin', sa.Boolean(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('users', 'is_admin')
    ### end Alembic commands ###



"""attribute_groups

Revision: 0064
Revision ID: 24191069ca2a
Revises: 2a74262efd7d
Create Date: 2016-06-14 11:48:39.567572

"""

# revision identifiers, used by Alembic.
revision = '24191069ca2a'
down_revision = '2a74262efd7d'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import select
from sqlalchemy.dialects import postgresql

card_models = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('attribute_group_id', sa.Integer),
    sa.Column('created_at', sa.DateTime)
)
card_model_attributes = sa.Table(
    'card_model_attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('card_model_id', sa.Integer),
    sa.Column('card_id', sa.Integer),
    sa.Column('group_id', sa.Integer)
)
cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('self_attribute_group_id', sa.Integer),
    sa.Column('created_at', sa.DateTime)
)

def upgrade():
    conn = op.get_bind()

    attribute_groups = op.create_table('attribute_groups',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('owner_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('shared_context', sa.PickleType(), nullable=True),
        sa.ForeignKeyConstraint(['owner_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_attribute_groups_owner_id'), 'attribute_groups', ['owner_id'], unique=False)

    op.add_column('card_model_attributes', sa.Column('group_id', sa.Integer(), nullable=True))
    op.create_index(op.f('ix_attributes_group_id'), 'card_model_attributes', ['group_id'], unique=False)
    op.create_foreign_key(None, 'card_model_attributes', 'attribute_groups', ['group_id'], ['id'], ondelete='SET NULL')

    op.add_column('card_models', sa.Column('attribute_group_id', sa.Integer(), nullable=True))
    op.create_index(op.f('ix_card_models_attribute_group_id'), 'card_models', ['attribute_group_id'], unique=False)
    op.create_foreign_key(None, 'card_models', 'attribute_groups', ['attribute_group_id'], ['id'], ondelete='SET NULL')

    op.add_column('cards', sa.Column('self_attribute_group_id', sa.Integer(), nullable=True))
    op.create_index(op.f('ix_cards_self_attribute_group_id'), 'cards', ['self_attribute_group_id'], unique=False)
    op.create_foreign_key(None, 'cards', 'attribute_groups', ['self_attribute_group_id'], ['id'], ondelete='SET NULL')

    q = select([card_models.c.id, card_models.c.owner_id, card_models.c.created_at])
    for model in conn.execute(q):
        r = conn.execute(attribute_groups.insert().values(
            owner_id=model.owner_id,
            created_at=model.created_at
        ))
        conn.execute(card_models.update().where(card_models.c.id==model.id).values(
            attribute_group_id=r.inserted_primary_key[0]
        ))
        conn.execute(card_model_attributes.update().where(card_model_attributes.c.card_model_id==model.id).values(
            group_id=r.inserted_primary_key[0]
        ))

    subq = select([card_model_attributes.c.card_id]).where(card_model_attributes.c.card_id != None)\
            .group_by(card_model_attributes.c.card_id)
    q = select([cards.c.id, cards.c.owner_id, cards.c.created_at]).where(cards.c.id.in_(subq))
    for card in conn.execute(q):
        r = conn.execute(attribute_groups.insert().values(
            owner_id=card.owner_id,
            created_at=card.created_at
        ))
        conn.execute(cards.update().where(cards.c.id==card.id).values(
            self_attribute_group_id=r.inserted_primary_key[0]
        ))
        conn.execute(card_model_attributes.update().where(card_model_attributes.c.card_id==card.id).values(
            group_id=r.inserted_primary_key[0]
        ))

    op.drop_index('ix_card_attributes_card_model_attribute_id', table_name='card_attributes')
    op.drop_constraint('card_attribute_card_model_attribute_id', 'card_attributes', type_='foreignkey')
    op.add_column('card_attributes', sa.Column('attribute_id', sa.Integer(), nullable=True))
    op.execute("UPDATE card_attributes SET attribute_id = card_model_attribute_id")

    op.drop_constraint('card_model_attributes_card_id_fkey', 'card_model_attributes', type_='foreignkey')
    op.drop_constraint('card_model_attributes_card_model_id_fkey', 'card_model_attributes', type_='foreignkey')
    op.rename_table("card_model_attributes", "attributes")

    op.create_index(op.f('ix_card_attributes_attribute_id'), 'card_attributes', ['attribute_id'], unique=False)
    op.create_foreign_key(None, 'card_attributes', 'attributes', ['attribute_id'], ['id'], ondelete='CASCADE')

    op.drop_column('card_attributes', 'card_model_attribute_id')
    op.drop_column('card_models', 'has_builtin_attributes')
    op.drop_index('ix_card_models_builtin', table_name='card_models')


def downgrade():
    pass


"""add-run-automation-role

Revision: 0230
Revision ID: 4748ed576de4
Revises: 9a7f12dce0ad
Create Date: 2022-03-02 07:35:35.760488

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '4748ed576de4'
down_revision = '9a7f12dce0ad'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE role_permissions SET actions = array_append(actions, 'RUN_AUTO_ACTIONS')
        FROM roles WHERE roles.id = role_permissions.role_id
        AND roles.builtin in ('ADMIN', 'MEMBER')
        AND role_permissions.object_type = 'project'
        AND role_permissions.object_id IS NULL
        AND NOT (role_permissions.actions @> '{"RUN_AUTO_ACTIONS"}'::varchar[])
    """)


def downgrade():
    pass



"""clear_perm_cache

Revision: 0159
Revision ID: 2f1187b05f65
Revises: 495c4d953e01
Create Date: 2019-02-15 16:52:40.387789

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '2f1187b05f65'
down_revision = '495c4d953e01'
branch_labels = None
depends_on = None


def upgrade():
    conn = op.get_bind()
    conn.execute('UPDATE users SET perm_cache_id = uuid_generate_v4() WHERE is_observer = TRUE')


def downgrade():
    pass



"""copied_from column in card_models

Revision: 0011
Revision ID: 37d254b2a1bc
Revises: 3a5f18e53496
Create Date: 2015-06-18 01:19:51.087792

"""

# revision identifiers, used by Alembic.
revision = '37d254b2a1bc'
down_revision = '5a7dc5f0f913'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('card_models', sa.Column('copied_from_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'card_models', 'card_models', ['copied_from_id'], ['id'], ondelete='SET NULL')
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, 'card_models', type_='foreignkey')
    op.drop_column('card_models', 'copied_from_id')
    ### end Alembic commands ###



"""missing_indexes

Revision: 0140
Revision ID: 25ae23668316
Revises: 537f032a290d
Create Date: 2018-06-25 19:18:38.163151

"""

# revision identifiers, used by Alembic.
revision = '25ae23668316'
down_revision = '537f032a290d'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_index(op.f('ix_users_pins_user_id_card_id'), 'users_pins', ['user_id', 'card_id'], unique=False)
    op.create_index(op.f('ix_project_webhooks_project_id_active'), 'project_webhooks', ['project_id', 'active'], unique=False)
    op.create_index(op.f('ix_organizations_is_user_org_created_by_id'), 'organizations', ['is_user_org', 'created_by_id'], unique=False)
    op.create_index(op.f('ix_projects_created_by_id'), 'projects', ['created_by_id'], unique=False)


def downgrade():
    op.drop_index(op.f('ix_users_pins_user_id_card_id'), table_name='users_pins')
    op.drop_index(op.f('ix_project_webhooks_project_id_active'), table_name='project_webhooks')
    op.drop_index(op.f('ix_organizations_is_user_org_created_by_id'), table_name='organizations')
    op.drop_index(op.f('ix_projects_created_by_id'), table_name='projects')



"""add_behavior_error

Revision: 0017
Revision ID: 26dc21c88e6
Revises: 302c2c3a9770
Create Date: 2015-08-12 15:11:48.168608

"""

# revision identifiers, used by Alembic.
revision = '26dc21c88e6'
down_revision = '302c2c3a9770'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('card_behaviors', sa.Column('error', sa.String(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('card_behaviors', 'error')
    ### end Alembic commands ###



"""requests_via_email_cc

Revision: 0167
Revision ID: 3cdc91fa1c35
Revises: 46d38eb71ef6
Create Date: 2019-06-21 10:43:58.002639

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '3cdc91fa1c35'
down_revision = '46d38eb71ef6'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('requests', sa.Column('via_email', sa.Boolean(), nullable=True))
    op.add_column('requests', sa.Column('email_cc', sa.String(), nullable=True))
    op.add_column('requests', sa.Column('email_subject', sa.String(), nullable=True))
    op.add_column('requests', sa.Column('email_last_message_id', sa.String(), nullable=True))
    op.add_column('requests', sa.Column('email_last_references', sa.String(), nullable=True))
    op.execute("UPDATE requests SET via_email = false")


def downgrade():
    op.drop_column('requests', 'via_email')
    op.drop_column('requests', 'email_cc')
    op.drop_column('requests', 'email_subject')
    op.drop_column('requests', 'email_last_message_id')
    op.drop_column('requests', 'email_last_references')



"""Reset groups cache

Revision: 0103
Revision ID: 575b3f3d895a
Revises: 14a5ced5c36c
Create Date: 2017-09-21 16:13:18.241319

"""

# revision identifiers, used by Alembic.
revision = '575b3f3d895a'
down_revision = '14a5ced5c36c'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.execute('UPDATE card_groups SET cache_id = uuid_generate_v4()')


def downgrade():
    op.execute('UPDATE card_groups SET cache_id = uuid_generate_v4()')



"""view_include_descendants

Revision: 0125
Revision ID: 2387660ed43b
Revises: 76df933e43b
Create Date: 2018-04-16 18:13:46.993383

"""

# revision identifiers, used by Alembic.
revision = '2387660ed43b'
down_revision = '76df933e43b'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('views', sa.Column('include_descendants', sa.Boolean(), nullable=True))
    op.execute("UPDATE views SET include_descendants = false")


def downgrade():
    op.drop_column('views', 'include_descendants')



"""billing_accounts

Revision: 0185
Revision ID: d3e47057f56
Revises: ebacad20699
Create Date: 2020-03-23 15:07:43.470616

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import select, func
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'd3e47057f56'
down_revision = 'ebacad20699'
branch_labels = None
depends_on = None


organizations = sa.Table(
    'organizations',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('created_by_id', sa.Integer(), nullable=True),
    sa.Column('created_at', sa.DateTime(), nullable=True),
    sa.Column('updated_at', sa.DateTime(), nullable=True),
    sa.Column('is_user_org', sa.Boolean(), nullable=True),
    sa.Column('billing_account_id', sa.Integer(), nullable=True),
    sa.Column('name', sa.String(), nullable=True),
    sa.Column('company', sa.String(), nullable=True),
    sa.Column('email', sa.String(), nullable=True),
    sa.Column('telephone', sa.String(), nullable=True),
    sa.Column('subscription_required', sa.Boolean(), nullable=True),
    sa.Column('trial_ends_at', sa.DateTime(), nullable=True),
    sa.Column('payment_method', sa.String(), nullable=True),
    sa.Column('prepaid_until', sa.DateTime(), nullable=True),
    sa.Column('can_self_export', sa.Boolean(), nullable=True),
    sa.Column('max_allowed_users', sa.Integer(), nullable=True),
    sa.Column('billing_address_city', sa.String(), nullable=True),
    sa.Column('billing_address_country', sa.String(), nullable=True),
    sa.Column('billing_address_line1', sa.String(), nullable=True),
    sa.Column('billing_address_line2', sa.String(), nullable=True),
    sa.Column('billing_address_state', sa.String(), nullable=True),
    sa.Column('billing_address_zip', sa.String(), nullable=True),
    sa.Column('billing_brand', sa.String(), nullable=True),
    sa.Column('billing_country', sa.String(), nullable=True),
    sa.Column('billing_exp_month', sa.String(), nullable=True),
    sa.Column('billing_exp_year', sa.String(), nullable=True),
    sa.Column('billing_last4', sa.String(), nullable=True),
    sa.Column('billing_name', sa.String(), nullable=True),
    sa.Column('eu_vat_country', sa.String(), nullable=True),
    sa.Column('eu_vat_number', sa.String(), nullable=True),
    sa.Column('eu_vat_rate', sa.Float(), nullable=True),
    sa.Column('has_stripe_source', sa.Boolean(), nullable=True),
    sa.Column('plan_last_charge_amount', sa.Float(), nullable=True),
    sa.Column('plan_last_charge_successful', sa.Boolean(), nullable=True),
    sa.Column('plan_last_charged_at', sa.DateTime(), nullable=True),
    sa.Column('plan_name', sa.String(), nullable=True),
    sa.Column('plan_next_charge_at', sa.DateTime(), nullable=True),
    sa.Column('plan_status', sa.String(), nullable=True),
    sa.Column('stripe_customer_id', sa.String(), nullable=True),
    sa.Column('stripe_subscription_id', sa.String(), nullable=True)
)

organization_members = sa.Table(
    'organization_members',
    sa.MetaData(),
    sa.Column('org_id', sa.Integer, primary_key=True),
    sa.Column('user_id', sa.Integer, primary_key=True)
)

invoices = sa.Table(
    'invoices',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('customer_id', sa.Integer, primary_key=True)
)


def upgrade():
    conn = op.get_bind()

    accounts = op.create_table('billing_accounts',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('company', sa.String(), nullable=True),
        sa.Column('payment_method', sa.String(), nullable=True),
        sa.Column('seat_count', sa.Integer(), nullable=True),
        sa.Column('auto_adjust_seat_count', sa.Boolean(), nullable=True),
        sa.Column('scheduled_seat_deletion', sa.Integer(), nullable=True),
        sa.Column('scheduled_seat_deletion_at', sa.DateTime(), nullable=True),
        sa.Column('billing_name', sa.String(), nullable=True),
        sa.Column('billing_email', sa.String(), nullable=True),
        sa.Column('billing_phone', sa.String(), nullable=True),
        sa.Column('billing_address_line1', sa.String(), nullable=True),
        sa.Column('billing_address_line2', sa.String(), nullable=True),
        sa.Column('billing_address_zip', sa.String(), nullable=True),
        sa.Column('billing_address_city', sa.String(), nullable=True),
        sa.Column('billing_address_state', sa.String(), nullable=True),
        sa.Column('billing_address_country', sa.String(), nullable=True),
        sa.Column('billing_type', sa.String(), nullable=True),
        sa.Column('billing_brand', sa.String(), nullable=True),
        sa.Column('billing_country', sa.String(), nullable=True),
        sa.Column('billing_exp_month', sa.String(), nullable=True),
        sa.Column('billing_exp_year', sa.String(), nullable=True),
        sa.Column('billing_last4', sa.String(), nullable=True),
        sa.Column('eu_vat_country', sa.String(), nullable=True),
        sa.Column('eu_vat_number', sa.String(), nullable=True),
        sa.Column('eu_vat_rate', sa.Float(), nullable=True),
        sa.Column('plan_name', sa.String(), nullable=True),
        sa.Column('plan_status', sa.String(), nullable=True),
        sa.Column('plan_current_period_start', sa.DateTime(), nullable=True),
        sa.Column('plan_current_period_end', sa.DateTime(), nullable=True),
        sa.Column('plan_cancel_at_period_end', sa.Boolean(), nullable=True),
        sa.Column('plan_last_invoice_at', sa.DateTime(), nullable=True),
        sa.Column('plan_last_invoice_amount', sa.Float(), nullable=True),
        sa.Column('plan_next_charge_at', sa.DateTime(), nullable=True),
        sa.Column('plan_cancelled_at', sa.DateTime(), nullable=True),
        sa.Column('plan_has_invoice_items', sa.Boolean(), nullable=True),
        sa.Column('plan_last_invoice_item_added_at', sa.DateTime(), nullable=True),
        sa.Column('stripe_customer_id', sa.String(), nullable=True),
        sa.Column('stripe_subscription_id', sa.String(), nullable=True),
        sa.Column('has_stripe_payment_method', sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_billing_accounts_stripe_customer_id'), 'billing_accounts', ['stripe_customer_id'], unique=False)
    op.create_index(op.f('ix_billing_accounts_stripe_subscription_id'), 'billing_accounts', ['stripe_subscription_id'], unique=False)
    op.create_index(op.f('ix_billing_accounts_has_stripe_payment_method'), 'billing_accounts', ['has_stripe_payment_method'], unique=False)
    op.create_index(op.f('ix_billing_accounts_plan_current_period_end'), 'billing_accounts', ['plan_current_period_end'], unique=False)
    op.create_index(op.f('ix_billing_accounts_plan_status'), 'billing_accounts', ['plan_status'], unique=False)
    op.create_index(op.f('ix_billing_accounts_invoice_items'), 'billing_accounts', ['plan_has_invoice_items', 'plan_last_invoice_item_added_at'], unique=False)

    op.add_column('organizations', sa.Column('billing_account_id', sa.Integer(), nullable=True))
    op.add_column('organizations', sa.Column('locked', sa.Boolean(), nullable=True))
    op.create_index(op.f('ix_organizations_billing_account_id'), 'organizations', ['billing_account_id'], unique=False)
    op.create_foreign_key('organizations_billing_account_id_fkey', 'organizations', 'billing_accounts', ['billing_account_id'], ['id'], ondelete='CASCADE')
    op.execute("UPDATE organizations SET locked = true, cache_id = generate_cache_id(id) WHERE plan_status = 'locked'")

    op.drop_constraint('invoices_customer_id_fkey', 'invoices')
    op.add_column('invoice_items', sa.Column('tax_amount', sa.Float(), nullable=True))
    op.add_column('invoice_items', sa.Column('tax_rate', sa.Float(), nullable=True))

    op.execute("""
        UPDATE invoice_items SET tax_rate = invoices.tax_rate FROM invoices WHERE invoices.id = invoice_items.invoice_id
    """)

    q = select([
        organizations,
        select([func.count(organization_members.c.user_id)]).where(
            sa.and_(organization_members.c.org_id==organizations.c.id, organization_members.c.user_id!=None)).label('nb_members')
    ]).where(sa.and_(organizations.c.subscription_required==True, organizations.c.is_user_org==False))

    for org in conn.execute(q):
        r = conn.execute(accounts.insert().values(
            created_by_id=org.created_by_id,
            created_at=org.created_at,
            updated_at=org.updated_at,
            name=org.name,
            company=org.company,
            payment_method='manual' if org.prepaid_until or org.payment_method != 'stripe' else 'stripe',
            seat_count=org.max_allowed_users if org.prepaid_until else org.nb_members,
            auto_adjust_seat_count=True,
            scheduled_seat_deletion=0,
            billing_name=org.billing_name,
            billing_email=org.email,
            billing_phone=org.telephone,
            billing_address_line1=org.billing_address_line1,
            billing_address_line2=org.billing_address_line2,
            billing_address_zip=org.billing_address_zip,
            billing_address_city=org.billing_address_city,
            billing_address_state=org.billing_address_state,
            billing_address_country=org.billing_address_country,
            billing_type='card',
            billing_brand=org.billing_brand,
            billing_country=org.billing_country,
            billing_exp_month=org.billing_exp_month,
            billing_exp_year=org.billing_exp_year,
            billing_last4=org.billing_last4,
            eu_vat_country=org.eu_vat_country,
            eu_vat_number=org.eu_vat_number,
            eu_vat_rate=org.eu_vat_rate,
            plan_name='kantree_team_monthly_eur' if org.plan_status == 'trialing' else ('kantree_eur' if not org.plan_name else org.plan_name),
            plan_status=org.plan_status,
            plan_current_period_start=org.plan_last_charged_at,
            plan_current_period_end=org.trial_ends_at if org.plan_status == 'trialing' else (org.prepaid_until if org.prepaid_until else org.plan_next_charge_at),
            plan_cancel_at_period_end=False,
            plan_last_invoice_amount=org.plan_last_charge_amount,
            plan_last_invoice_at=org.plan_last_charged_at,
            plan_next_charge_at=org.plan_next_charge_at,
            plan_has_invoice_items=False,
            stripe_customer_id=org.stripe_customer_id,
            stripe_subscription_id=org.stripe_subscription_id,
            has_stripe_payment_method=org.has_stripe_source
        ))
        account_id = r.inserted_primary_key[0]
        conn.execute(organizations.update().where(organizations.c.id==org.id)\
            .values(billing_account_id=account_id))
        conn.execute(invoices.update().where(invoices.c.customer_id==org.id)\
            .values(customer_id=account_id))

    conn.execute(invoices.update().where(sa.and_(invoices.c.customer_id==organizations.c.id, ~organizations.c.subscription_required))\
        .values(customer_id=None))

    op.create_foreign_key('invoices_customer_id_fkey', 'invoices', 'billing_accounts', ['customer_id'], ['id'], ondelete='SET NULL')

def downgrade():
    pass



"""org_desc_and_featured_items

Revision: 0187
Revision ID: 24858bb4f862
Revises: 416d8e4d464b
Create Date: 2020-05-28 22:06:20.170593

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '24858bb4f862'
down_revision = '416d8e4d464b'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('organizations', sa.Column('description', sa.Text(), nullable=True))

    op.create_table('organization_featured_items',
        sa.Column('id', sa.Integer(), primary_key=True),
        sa.Column('org_id', sa.Integer(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('object_type', sa.String(), nullable=True),
        sa.Column('object_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['org_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_organization_featured_items_org_id'), 'organization_featured_items', ['org_id'], unique=False)
    op.create_unique_constraint("organization_featured_items_object_unique", "organization_featured_items", ["org_id", "object_type", "object_id"])


def downgrade():
    op.drop_column('organizations', 'description')
    op.drop_table('organization_featured_items')



"""add_reminders

Revision: 0141
Revision ID: 1d830ee99d4c
Revises: 25ae23668316
Create Date: 2018-05-09 15:14:15.912273

"""

# revision identifiers, used by Alembic.
revision = '1d830ee99d4c'
down_revision = '25ae23668316'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
import uuid
import datetime

def upgrade():
    conn = op.get_bind()

    reminders = op.create_table('reminders',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('targets', postgresql.JSONB(astext_type=sa.Text()), nullable=True),
        sa.Column('reminding_date', sa.DateTime(), nullable=True),
        sa.Column('attribute_id', sa.Integer(), nullable=True),
        sa.Column('cache_id', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['attribute_id'], ['attributes.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )

    r_infos = conn.execute("""
        SELECT c.id, a.attribute_id, a.value->'end' "end", a.created_at, a.created_by_id
        FROM cards c
        JOIN card_attributes a ON a.card_id = c.id
        WHERE a.type_name = 'date'
            AND a.value is NOT NULL
            AND (a.value->'end')::text != 'null'
            AND to_timestamp(substring((a.value->'end')::text from 2 for 10), 'YYYY-MM-DD') >= now() + INTERVAL '1 day'
    """)

    to_insert = []
    for r in r_infos:
        date = r[2]
        if len(date) == 10:
            date += ' 00:00:00'
        to_insert.append({
            'created_at': r[3],
            'created_by_id': r[4],
            'card_id': r[0],
            'targets': ['subscribers'],
            'reminding_date': datetime.datetime.strptime(date[:19], '%Y-%m-%d %H:%M:%S') - datetime.timedelta(days=1),
            'attribute_id': r[1],
            'cache_id': uuid.uuid4()
        })
    op.bulk_insert(reminders, to_insert)

    op.execute("""
        UPDATE role_permissions SET actions = array_append(actions, 'MANAGE_REMINDERS')
        FROM roles WHERE roles.id = role_permissions.role_id
        AND roles.builtin = 'ADMIN'
        AND role_permissions.object_type = 'project'
        AND role_permissions.object_id IS NULL
        AND NOT (role_permissions.actions @> '{"MANAGE_REMINDERS"}'::varchar[])
    """)

    op.execute("""
        UPDATE roles SET cache_id = uuid_generate_v4()
        WHERE roles.id IN (
            SELECT DISTINCT roles.id
            FROM roles, role_permissions
            WHERE role_permissions.role_id = roles.id
            AND roles.builtin = 'ADMIN'
            AND role_permissions.object_type = 'project'
            AND role_permissions.object_id IS NULL
            AND (role_permissions.actions @> '{"MANAGE_REMINDERS"}'::varchar[])
        )
    """)

    op.execute("""
        UPDATE projects SET members_cache_id = uuid_generate_v4()
    """)

    op.add_column('projects', sa.Column('send_emails', sa.Boolean(), nullable=True, server_default="True"))

def downgrade():
    op.drop_column('projects', 'send_emails')
    op.drop_table('reminders')



"""remove_daterange

Revision: 0088
Revision ID: 57c60056d090
Revises: 4e8995b8a6a6
Create Date: 2017-03-15 09:36:44.212943

"""

# revision identifiers, used by Alembic.
revision = '57c60056d090'
down_revision = '4e8995b8a6a6'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        update card_attributes set value = json_build_object('end', value->>0)::jsonb from attributes
        where attributes.id = card_attributes.attribute_id and attributes.type_name = 'date'
    """)
    op.execute("""update attributes set type_name = 'date' where type_name = 'daterange'""")
    op.execute("""update card_attributes set type_name = 'date' where type_name = 'daterange'""")


def downgrade():
    pass



"""user_onboarded_as_array

Revision: 0191
Revision ID: 16a37635c0ed
Revises: f6c9987ca2e6
Create Date: 2020-08-31 14:42:54.853154

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '16a37635c0ed'
down_revision = 'f6c9987ca2e6'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("DROP VIEW IF EXISTS users_report_view")
    op.execute("ALTER TABLE users RENAME COLUMN onboarded TO onboarded_old")

    op.add_column('users', sa.Column('onboarded', postgresql.ARRAY(sa.String()), nullable=True))
    op.add_column('users', sa.Column('preferred_template_library_tag', sa.String(), nullable=True))

    op.execute("""UPDATE users SET onboarded = '{"user", "project"}'::text[] WHERE onboarded_old""")
    op.drop_column('users', 'onboarded_old')

    op.execute("""
        CREATE OR REPLACE VIEW users_report_view AS
        SELECT
            id,
            signup_at,
            signup_from,
            signup_provider,
            auth_providers,
            last_login_at,
            last_login_from,
            last_login_provider,
            split_part(email, '@', 2) email_domain,
            can_create_organizations,
            locale,
            timezone,
            enable_email_notifications,
            updated_at,
            is_admin,
            subscription_required,
            trial_ends_at,
            can_join_organizations,
            is_observer,
            is_disabled,
            disabled_at,
            last_activity_at,
            onboarded,
            last_password_change_at,
            last_terms_validation,
            email_newsletter,
            email_notifications,
            is_super_admin,
            signup_country,
            signup_cpc_source,
            signup_cpc_campaign,
            signup_cpc_ad,
            signup_cpc_keyword,
            signup_cpc_sessionid,
            last_access_at,
            last_password_change_from,
            is_auto_disabled,
            two_factor_auth_enabled
        FROM users
    """)


def downgrade():
    pass



"""Init db

Revision: 0000
Revision ID: d45177564c9
Revises: 
Create Date: 2015-06-01 17:42:30.683311

"""

# revision identifiers, used by Alembic.
revision = 'd45177564c9'
down_revision = None
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.create_table('users',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('signup_at', sa.DateTime(), nullable=True),
        sa.Column('signup_from', sa.String(), nullable=True),
        sa.Column('signup_provider', sa.String(), nullable=True),
        sa.Column('auth_providers', postgresql.ARRAY(sa.String()), nullable=True),
        sa.Column('last_login_at', sa.DateTime(), nullable=True),
        sa.Column('last_login_from', sa.String(), nullable=True),
        sa.Column('last_login_provider', sa.String(), nullable=True),
        sa.Column('username', sa.String(), nullable=True),
        sa.Column('email', sa.String(), nullable=True),
        sa.Column('password', sa.String(), nullable=True),
        sa.Column('avatar_filename', sa.String(), nullable=True),
        sa.Column('unread_notifications', sa.Integer(), nullable=True),
        sa.Column('trello_oauth_token', sa.String(), nullable=True),
        sa.Column('trello_oauth_token_secret', sa.String(), nullable=True),
        sa.Column('trello_user_id', sa.String(), nullable=True),
        sa.Column('trello_username', sa.String(), nullable=True),
        sa.Column('github_access_token', sa.String(), nullable=True),
        sa.Column('github_username', sa.String(), nullable=True),
        sa.Column('github_id', sa.String(), nullable=True),
        sa.Column('github_email', sa.String(), nullable=True),
        sa.Column('asana_token', sa.String(), nullable=True),
        sa.Column('asana_name', sa.String(), nullable=True),
        sa.Column('asana_id', sa.String(), nullable=True),
        sa.Column('asana_email', sa.String(), nullable=True),
        sa.Column('can_create_organizations', sa.Boolean(), nullable=True),
        sa.Column('locale', sa.String(), nullable=True),
        sa.Column('timezone', sa.String(), nullable=True),
        sa.Column('currency', sa.String(), nullable=True),
        sa.Column('enable_email_notifications', sa.Boolean(), nullable=True),
        sa.Column('user_org_id', sa.Integer(), nullable=True),
        sa.Column('is_special_user', sa.Boolean(), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('email'),
        sa.UniqueConstraint('username')
    )
    op.create_index(op.f('ix_users_asana_id'), 'users', ['asana_id'], unique=False)
    op.create_index(op.f('ix_users_github_id'), 'users', ['github_id'], unique=False)
    op.create_index(op.f('ix_users_is_special_user'), 'users', ['is_special_user'], unique=False)
    op.create_index(op.f('ix_users_trello_user_id'), 'users', ['trello_user_id'], unique=False)

    op.create_table('organizations',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('is_user_org', sa.Boolean(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('default_card_model_id', sa.Integer(), nullable=True),
        sa.Column('can_invite_guest_collabs', sa.Boolean(), nullable=True),
        sa.Column('can_create_private_boards', sa.Boolean(), nullable=True),
        sa.Column('trial_ends_at', sa.DateTime(), nullable=True),
        sa.Column('status', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_foreign_key(None, 'users', 'organizations', ['user_org_id'], ['id'])

    op.create_table('organization_members',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('org_id', sa.Integer(), nullable=True),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('permission', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['org_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )

    op.create_table('api_keys',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('value', sa.String(), nullable=True),
        sa.Column('last_accessed_at', sa.DateTime(), nullable=True),
        sa.Column('last_accessed_from', sa.String(), nullable=True),
        sa.Column('expires_at', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('value')
    )

    op.create_table('board_templates',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('owner_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('builtin', sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(['owner_id'], ['organizations.id'], ),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_board_templates_builtin'), 'board_templates', ['builtin'], unique=False)
    op.create_index(op.f('ix_board_templates_name'), 'board_templates', ['name'], unique=False)

    op.create_table('board_template_lists',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('board_template_id', sa.Integer(), nullable=True),
        sa.Column('title', sa.String(), nullable=True),
        sa.Column('bgcolor', sa.String(length=7), nullable=True),
        sa.Column('show_flow_indicator', sa.Boolean(), nullable=True),
        sa.Column('mark_card_as_done', sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(['board_template_id'], ['board_templates.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )

    op.create_table('card_models',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('owner_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('builtin', sa.Boolean(), nullable=True),
        sa.Column('listable', sa.Boolean(), nullable=True),
        sa.Column('available_for_all', sa.Boolean(), nullable=True),
        sa.Column('extends_id', sa.Integer(), nullable=True),
        sa.Column('board_template_id', sa.Integer(), nullable=True),
        sa.Column('children_default_model_id', sa.Integer(), nullable=True),
        sa.Column('indicator', sa.String(), nullable=True),
        sa.Column('card_color', sa.String(length=7), nullable=True),
        sa.Column('created_in_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['board_template_id'], ['board_templates.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['owner_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_models_builtin'), 'card_models', ['builtin'], unique=False)
    op.create_index(op.f('ix_card_models_name'), 'card_models', ['name'], unique=False)
    op.create_foreign_key(None, 'card_models', 'card_models', ['extends_id'], ['id'], ondelete='SET NULL')
    op.create_foreign_key(None, 'card_models', 'card_models', ['children_default_model_id'], ['id'], ondelete='SET NULL')
    op.create_foreign_key(None, 'organizations', 'card_models', ['default_card_model_id'], ['id'], ondelete='SET NULL')

    op.create_table('card_model_attributes',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('card_model_id', sa.Integer(), nullable=True),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('title', sa.String(), nullable=True),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('type_name', sa.String(), nullable=True),
        sa.Column('container', sa.String(), nullable=True),
        sa.Column('position', sa.Integer(), nullable=True),
        sa.Column('options', sa.PickleType(), nullable=True),
        sa.Column('default_value', sa.PickleType(), nullable=True),
        sa.ForeignKeyConstraint(['card_model_id'], ['card_models.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_model_attributes_position'), 'card_model_attributes', ['position'], unique=False)
    op.create_index(op.f('ix_card_model_attributes_type_name'), 'card_model_attributes', ['type_name'], unique=False)

    op.create_table('cards',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('owner_id', sa.Integer(), nullable=True),
        sa.Column('parent_id', sa.Integer(), nullable=True),
        sa.Column('parents_ids', postgresql.ARRAY(sa.Integer()), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('is_link', sa.Boolean(), nullable=True),
        sa.Column('link_to_id', sa.Integer(), nullable=True),
        sa.Column('link_has_own_attributes', sa.Boolean(), nullable=True),
        sa.Column('ref', sa.Integer(), nullable=True),
        sa.Column('ref_counter', sa.Integer(), nullable=True),
        sa.Column('title', sa.String(), nullable=True),
        sa.Column('slug', sa.String(), nullable=True),
        sa.Column('model_id', sa.Integer(), nullable=True),
        sa.Column('default_child_card_model_id', sa.Integer(), nullable=True),
        sa.Column('list_id', sa.Integer(), nullable=True),
        sa.Column('list_position', sa.Integer(), nullable=True),
        sa.Column('public_permission', sa.Integer(), nullable=True),
        sa.Column('org_members_can_observe', sa.Boolean(), nullable=True),
        sa.Column('org_members_can_autojoin', sa.Boolean(), nullable=True),
        sa.Column('default_org_member_permission', sa.Integer(), nullable=True),
        sa.Column('is_archived', sa.Boolean(), nullable=True),
        sa.Column('is_list_archived', sa.Boolean(), nullable=True),
        sa.Column('is_parent_archived', sa.Integer(), nullable=True),
        sa.Column('archived_at', sa.DateTime(), nullable=True),
        sa.Column('meta', postgresql.JSONB(), nullable=True),
        sa.Column('children_attributes_ctx', sa.PickleType(), nullable=True),
        sa.Column('cache_id', sa.String(), nullable=True),
        sa.Column('board_bgcolor', sa.String(), nullable=True),
        sa.Column('done', sa.Boolean(), nullable=True),
        sa.Column('done_at', sa.DateTime(), nullable=True),
        sa.Column('reopened', sa.Boolean(), nullable=True),
        sa.Column('milestone_id', sa.Integer(), nullable=True),
        sa.Column('image_cover_url', sa.String(), nullable=True),
        sa.Column('board_created', sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['default_child_card_model_id'], ['card_models.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['link_to_id'], ['cards.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['model_id'], ['card_models.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['owner_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('slug')
    )
    op.create_index(op.f('ix_cards_archived_at'), 'cards', ['archived_at'], unique=False)
    op.create_index(op.f('ix_cards_board_created'), 'cards', ['board_created'], unique=False)
    op.create_index(op.f('ix_cards_done'), 'cards', ['done'], unique=False)
    op.create_index(op.f('ix_cards_done_at'), 'cards', ['done_at'], unique=False)
    op.create_index(op.f('ix_cards_is_archived'), 'cards', ['is_archived'], unique=False)
    op.create_index(op.f('ix_cards_is_list_archived'), 'cards', ['is_list_archived'], unique=False)
    op.create_index(op.f('ix_cards_is_parent_archived'), 'cards', ['is_parent_archived'], unique=False)
    op.create_index(op.f('ix_cards_list_position'), 'cards', ['list_position'], unique=False)
    op.create_index(op.f('ix_cards_meta'), 'cards', ['meta'], unique=False)
    op.create_index(op.f('ix_cards_parents_ids'), 'cards', ['parents_ids'], unique=False)
    op.create_index(op.f('ix_cards_ref'), 'cards', ['ref'], unique=False)
    op.create_index(op.f('ix_cards_reopened'), 'cards', ['reopened'], unique=False)
    op.create_foreign_key(None, 'cards', 'cards', ['parent_id'], ['id'], ondelete='CASCADE')
    op.create_foreign_key(None, 'card_models', 'cards', ['created_in_id'], ['id'], ondelete='SET NULL')
    op.create_foreign_key(None, 'card_model_attributes', 'cards', ['card_id'], ['id'], ondelete='CASCADE')

    op.create_table('card_subscriptions',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('auto', sa.Boolean(), nullable=True),
        sa.Column('send_emails', sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )

    op.create_table('card_attributes',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('type_name', sa.String(), nullable=True),
        sa.Column('value', postgresql.JSONB(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_attributes_name'), 'card_attributes', ['name'], unique=False)
    op.create_index(op.f('ix_card_attributes_type_name'), 'card_attributes', ['type_name'], unique=False)
    op.create_index(op.f('ix_card_attributes_value'), 'card_attributes', ['value'], unique=False)

    op.create_table('card_behaviors',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('enabled_by_id', sa.Integer(), nullable=True),
        sa.Column('enabled_at', sa.DateTime(), nullable=True),
        sa.Column('name', sa.String(), nullable=True),
        sa.Column('options', postgresql.JSONB(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['enabled_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_behaviors_name'), 'card_behaviors', ['name'], unique=False)

    op.create_table('card_enabled_models',
        sa.Column('model_id', sa.Integer(), nullable=True),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['model_id'], ['card_models.id'], ondelete='CASCADE')
    )

    op.create_table('card_lists',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('parent_id', sa.Integer(), nullable=True),
        sa.Column('title', sa.String(), nullable=True),
        sa.Column('parents_ids', postgresql.ARRAY(sa.Integer()), nullable=True),
        sa.Column('position', sa.Integer(), nullable=True),
        sa.Column('is_archived', sa.Boolean(), nullable=True),
        sa.Column('is_parent_archived', sa.Integer(), nullable=True),
        sa.Column('archived_at', sa.DateTime(), nullable=True),
        sa.Column('meta', sa.PickleType(), nullable=True),
        sa.Column('bgcolor', sa.String(), nullable=True),
        sa.Column('show_flow_indicator', sa.Boolean(), nullable=True),
        sa.Column('mark_card_as_done', sa.Boolean(), nullable=True),
        sa.Column('default_card_model_id', sa.Integer(), nullable=True),
        sa.Column('auto_assign_user', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['default_card_model_id'], ['card_models.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['parent_id'], ['cards.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_card_lists_archived_at'), 'card_lists', ['archived_at'], unique=False)
    op.create_index(op.f('ix_card_lists_is_archived'), 'card_lists', ['is_archived'], unique=False)
    op.create_index(op.f('ix_card_lists_parents_ids'), 'card_lists', ['parents_ids'], unique=False)
    op.create_index(op.f('ix_card_lists_position'), 'card_lists', ['position'], unique=False)
    op.create_foreign_key(None, 'cards', 'card_lists', ['list_id'], ['id'], ondelete='CASCADE')

    op.create_table('card_members',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('permission', sa.Integer(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )

    op.create_table('milestones',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('parent_id', sa.Integer(), nullable=True),
        sa.Column('ref', sa.Integer(), nullable=True),
        sa.Column('title', sa.String(), nullable=True),
        sa.Column('start_date', sa.DateTime(), nullable=True),
        sa.Column('end_date', sa.DateTime(), nullable=True),
        sa.Column('color', sa.String(length=7), nullable=True),
        sa.Column('closed', sa.Boolean(), nullable=True),
        sa.Column('cache_id', sa.String(), nullable=True),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_milestones_closed'), 'milestones', ['closed'], unique=False)
    op.create_index(op.f('ix_milestones_end_date'), 'milestones', ['end_date'], unique=False)
    op.create_index(op.f('ix_milestones_ref'), 'milestones', ['ref'], unique=False)
    op.create_foreign_key(None, 'cards', 'milestones', ['milestone_id'], ['id'], ondelete='SET NULL')
    op.create_foreign_key(None, 'milestones', 'cards', ['parent_id'], ['id'], ondelete='CASCADE')

    op.create_table('stream',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('item_type', sa.String(length=10), nullable=True),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('message', sa.Text(), nullable=True),
        sa.Column('is_pinned', sa.Boolean(), nullable=True),
        sa.Column('label', sa.String(), nullable=True),
        sa.Column('subject_type', sa.String(), nullable=True),
        sa.Column('subject_id', sa.Integer(), nullable=True),
        sa.Column('subject_value', sa.PickleType(), nullable=True),
        sa.Column('args', sa.PickleType(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_stream_is_pinned'), 'stream', ['is_pinned'], unique=False)
    op.create_index(op.f('ix_stream_item_type'), 'stream', ['item_type'], unique=False)

    op.create_table('invites',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('created_by_id', sa.Integer(), nullable=True),
        sa.Column('email', sa.String(), nullable=True),
        sa.Column('org_id', sa.Integer(), nullable=True),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.Column('params', sa.PickleType(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['created_by_id'], ['users.id'], ondelete='SET NULL'),
        sa.ForeignKeyConstraint(['org_id'], ['organizations.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_invites_email'), 'invites', ['email'], unique=False)

    op.create_table('users_pins',
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('card_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['card_id'], ['cards.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['user_id'], ['users.id'], ondelete='CASCADE')
    )


def downgrade():
    op.drop_table('users_pins')
    op.drop_index(op.f('ix_users_trello_user_id'), table_name='users')
    op.drop_index(op.f('ix_users_is_special_user'), table_name='users')
    op.drop_index(op.f('ix_users_github_id'), table_name='users')
    op.drop_index(op.f('ix_users_asana_id'), table_name='users')
    op.drop_table('users')
    op.drop_index(op.f('ix_stream_item_type'), table_name='stream')
    op.drop_index(op.f('ix_stream_is_pinned'), table_name='stream')
    op.drop_table('stream')
    op.drop_table('organization_members')
    op.drop_table('organizations')
    op.drop_index(op.f('ix_milestones_ref'), table_name='milestones')
    op.drop_index(op.f('ix_milestones_end_date'), table_name='milestones')
    op.drop_index(op.f('ix_milestones_closed'), table_name='milestones')
    op.drop_table('milestones')
    op.drop_index(op.f('ix_invites_email'), table_name='invites')
    op.drop_table('invites')
    op.drop_index(op.f('ix_cards_reopened'), table_name='cards')
    op.drop_index(op.f('ix_cards_ref'), table_name='cards')
    op.drop_index(op.f('ix_cards_parents_ids'), table_name='cards')
    op.drop_index(op.f('ix_cards_meta'), table_name='cards')
    op.drop_index(op.f('ix_cards_list_position'), table_name='cards')
    op.drop_index(op.f('ix_cards_is_parent_archived'), table_name='cards')
    op.drop_index(op.f('ix_cards_is_list_archived'), table_name='cards')
    op.drop_index(op.f('ix_cards_is_archived'), table_name='cards')
    op.drop_index(op.f('ix_cards_done_at'), table_name='cards')
    op.drop_index(op.f('ix_cards_done'), table_name='cards')
    op.drop_index(op.f('ix_cards_board_created'), table_name='cards')
    op.drop_index(op.f('ix_cards_archived_at'), table_name='cards')
    op.drop_table('cards')
    op.drop_table('card_subscriptions')
    op.drop_index(op.f('ix_card_models_name'), table_name='card_models')
    op.drop_index(op.f('ix_card_models_builtin'), table_name='card_models')
    op.drop_table('card_models')
    op.drop_index(op.f('ix_card_model_attributes_type_name'), table_name='card_model_attributes')
    op.drop_index(op.f('ix_card_model_attributes_position'), table_name='card_model_attributes')
    op.drop_table('card_model_attributes')
    op.drop_table('card_members')
    op.drop_index(op.f('ix_card_lists_position'), table_name='card_lists')
    op.drop_index(op.f('ix_card_lists_parents_ids'), table_name='card_lists')
    op.drop_index(op.f('ix_card_lists_is_archived'), table_name='card_lists')
    op.drop_index(op.f('ix_card_lists_archived_at'), table_name='card_lists')
    op.drop_table('card_lists')
    op.drop_table('card_enabled_models')
    op.drop_index(op.f('ix_card_behaviors_name'), table_name='card_behaviors')
    op.drop_table('card_behaviors')
    op.drop_index(op.f('ix_card_attributes_value'), table_name='card_attributes')
    op.drop_index(op.f('ix_card_attributes_type_name'), table_name='card_attributes')
    op.drop_index(op.f('ix_card_attributes_name'), table_name='card_attributes')
    op.drop_table('card_attributes')
    op.drop_index(op.f('ix_board_templates_name'), table_name='board_templates')
    op.drop_index(op.f('ix_board_templates_builtin'), table_name='board_templates')
    op.drop_table('board_templates')
    op.drop_table('board_template_lists')
    op.drop_table('api_keys')



"""user_terms_validation

Revision: 0131
Revision ID: 3feaab6b3df
Revises: 384360aef2d6
Create Date: 2018-05-22 18:52:22.623289

"""

# revision identifiers, used by Alembic.
revision = '3feaab6b3df'
down_revision = '384360aef2d6'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('last_terms_validation', sa.DateTime(), nullable=True))
    op.add_column('users', sa.Column('email_newsletter', sa.Boolean(), nullable=True))
    op.add_column('users', sa.Column('email_notifications', sa.Boolean(), nullable=True))
    op.execute("UPDATE users SET email_newsletter = false, email_notifications = enable_emails")


def downgrade():
    op.drop_column('users', 'last_terms_validation')
    op.drop_column('users', 'email_newsletter')
    op.drop_column('users', 'email_notifications')



"""group_validation_rule

Revision: 0084
Revision ID: 427a0e562031
Revises: 57af91cc668a
Create Date: 2017-02-16 11:30:56.009837

"""

# revision identifiers, used by Alembic.
revision = '427a0e562031'
down_revision = '57af91cc668a'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('card_groups', sa.Column('validation_rule', sa.String(), nullable=True))
    op.create_index(op.f('ix_card_groups_validation_rule'), 'card_groups', ['validation_rule'], unique=False)


def downgrade():
    op.drop_index(op.f('ix_card_groups_validation_rule'), table_name='card_groups')
    op.drop_column('card_groups', 'validation_rule')



"""not_null_attr_position_ns

Revision: 0243
Revision ID: aa02a8f6f3b1
Revises: 3769937df895
Create Date: 2022-09-15 09:49:27.145744

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'aa02a8f6f3b1'
down_revision = '3769937df895'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("delete from attribute_positions where namespace is null")
    op.alter_column('attribute_positions', 'namespace', nullable=False)


def downgrade():
    pass



"""update_views_filter_due_week

Revision: 0139
Revision ID: 537f032a290d
Revises: 4e52740ad490
Create Date: 2018-07-26 16:07:29.309229

"""

# revision identifiers, used by Alembic.
revision = '537f032a290d'
down_revision = '4e52740ad490'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        UPDATE views SET filter = 'date?(week)' WHERE filter = 'due:week'
    """)


def downgrade():
    pass



"""added username_lcase

Revision: 0008
Revision ID: 2075f8467bf0
Revises: 4ce0659a41fd
Create Date: 2015-06-16 23:04:28.403511

"""

# revision identifiers, used by Alembic.
revision = '2075f8467bf0'
down_revision = '4ce0659a41fd'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('users', sa.Column('username_lcase', sa.String(), nullable=True))
    op.drop_constraint('users_username_key', 'users', type_='unique')
    op.create_unique_constraint(None, 'users', ['username_lcase'])
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, 'users', type_='unique')
    op.create_unique_constraint('users_username_key', 'users', ['username'])
    op.drop_column('users', 'username_lcase')
    ### end Alembic commands ###



"""group_type_display

Revision: 0032
Revision ID: 21a6d541bc0f
Revises: 3db4ee6d1d77
Create Date: 2015-11-11 15:20:54.101344

"""

# revision identifiers, used by Alembic.
revision = '21a6d541bc0f'
down_revision = '3db4ee6d1d77'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


enum_type = sa.Enum('label', 'border', name='card_group_type_display')

def upgrade():
    enum_type.create(op.get_bind(), checkfirst=False)
    op.alter_column('card_group_types', 'display_on_card', new_column_name='display_on_card2')
    op.add_column('card_group_types', sa.Column('display_on_card', enum_type, nullable=True))
    op.execute("UPDATE card_group_types SET display_on_card = 'label' WHERE display_on_card2")
    op.drop_column('card_group_types', 'display_on_card2')
    op.add_column('card_group_types', sa.Column('auto_random_color', sa.Boolean(), nullable=True))


def downgrade():
    op.drop_column('card_group_types', 'display_on_card')
    op.drop_column('card_group_types', 'auto_random_color')
    op.add_column('card_group_types', sa.Column('display_on_card', sa.Boolean(), nullable=True))
    enum_type.drop(op.get_bind(), checkfirst=False)


"""search

Revision: 0083
Revision ID: 57af91cc668a
Revises: 3ce9106c2680
Create Date: 2017-01-24 15:24:32.766888

"""

# revision identifiers, used by Alembic.
revision = '57af91cc668a'
down_revision = '3ce9106c2680'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.create_index(op.f('ix_attributes_name_lower'), 'attributes', [sa.text('lower(name)')], unique=False)
    op.create_index(op.f('ix_card_models_name_lower'), 'card_models', [sa.text('lower(name)')], unique=False)
    op.create_index(op.f('ix_organizations_name_lower'), 'organizations', [sa.text('lower(name)')], unique=False)

    op.execute("""CREATE EXTENSION IF NOT EXISTS unaccent;""")
    op.execute("""
        CREATE OR REPLACE FUNCTION build_tsvector(content text) RETURNS tsvector
        AS
        $BODY$
            SELECT to_tsvector(unaccent(content));
        $BODY$
        LANGUAGE sql
        IMMUTABLE;
    """)

    op.execute("""CREATE INDEX ix_cards_fulltext ON cards USING gin(build_tsvector(title))""")
    op.execute("""CREATE INDEX ix_card_attributes_fulltext ON card_attributes USING gin(build_tsvector(unindexed_value))""")

    op.execute("""UPDATE views SET filter = regexp_replace(filter, 'due:([a-zA-Z0-9]+)', E'date?("\\\\1")', 'g')""")
    op.execute("""UPDATE views SET filter = regexp_replace(filter, 'state:(todo|doing|done)', E'state=\\\\1', 'g')""")
    op.execute("""UPDATE views SET filter = regexp_replace(filter, 'model:([a-zA-Z0-9]+)', E'model=\\\\1', 'g')""")
    op.execute("""UPDATE views SET filter = replace(filter, '#!active', 'is_in_active_group?()')""")


def downgrade():
    op.drop_index(op.f('ix_attributes_name_lower'), table_name='attributes')
    op.drop_index(op.f('ix_card_models_name_lower'), table_name='card_models')
    op.drop_index(op.f('ix_organizations_name_lower'), table_name='organizations')
    op.drop_index(op.f('ix_cards_fulltext'), table_name='cards')
    op.drop_index(op.f('ix_card_attributes_fulltext'), table_name='card_attributes')
    op.execute("DROP FUNCTION build_tsvector(text)")
    op.execute("DROP EXTENSION unaccent")


"""lock_view_edition

Revision ID: 0270
Revises: 0269
Create Date: 2025-02-07 09:11:53.319423

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0270'
down_revision = '0269'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('views', sa.Column('locked', sa.Boolean(), nullable=False, server_default="false"))


def downgrade():
    op.drop_column('views', 'locked')



"""add_cardlists_desc

Revision: 0012
Revision ID: 48b6284b245e
Revises: 37d254b2a1bc
Create Date: 2015-07-07 09:55:31.583624

"""

# revision identifiers, used by Alembic.
revision = '48b6284b245e'
down_revision = '37d254b2a1bc'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('card_lists', sa.Column('description', sa.Text(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('card_lists', 'description')
    ### end Alembic commands ###



"""clear_attributes_cache

Revision ID: 0281
Revises: 0280
Create Date: 2025-10-30 08:56:12.178870

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '0281'
down_revision = '0280'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""UPDATE cards SET cache_id = uuid_generate_v4() FROM card_attributes WHERE card_attributes.card_id = cards.id AND card_attributes.type_name = 'relationships'""")
    op.execute("""UPDATE card_models SET cache_id = uuid_generate_v4() FROM attribute_groups ag JOIN attributes attr ON attr.group_id = ag.id WHERE ag.id = card_models.attribute_group_id AND attr.type_name = 'relationships'""")
    pass


def downgrade():
    pass



"""build_tsvector

Revision: 0188
Revision ID: 68d34c581b25
Revises: 24858bb4f862
Create Date: 2020-08-13 11:36:05.800234

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '68d34c581b25'
down_revision = '24858bb4f862'
branch_labels = None
depends_on = None


def upgrade():
    # some old installations may not have the good build_tsvector() function
    # we need to make sure that the schema is mentionned in unaccent() under postgresql12

    op.execute("""
        CREATE OR REPLACE FUNCTION build_tsvector(content text) RETURNS tsvector
        AS
        $BODY$
            SELECT to_tsvector(public.unaccent('public.unaccent', left(content, 500000)));
        $BODY$
        LANGUAGE sql
        IMMUTABLE;
    """)

    op.execute("""DROP INDEX IF EXISTS ix_cards_fulltext""")
    op.execute("""DROP INDEX IF EXISTS ix_card_attributes_fulltext""")
    op.execute("""DROP INDEX IF EXISTS ix_card_comments_fulltext""")

    op.execute("""CREATE INDEX ix_cards_fulltext ON cards USING gin(build_tsvector(title))""")
    op.execute("""CREATE INDEX ix_card_attributes_fulltext ON card_attributes USING gin(build_tsvector(unindexed_value))""")
    op.execute("""CREATE INDEX ix_card_comments_fulltext ON card_comments USING gin(build_tsvector(message))""")


def downgrade():
    pass



"""google_token_expiry

Revision: 0128
Revision ID: 26834f77e639
Revises: 4e8938a1c66
Create Date: 2018-04-28 15:49:37.396271

"""

# revision identifiers, used by Alembic.
revision = '26834f77e639'
down_revision = '4e8938a1c66'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('users', sa.Column('google_token_expiry', sa.DateTime(), nullable=True))


def downgrade():
    op.drop_column('users', 'google_token_expiry')



"""clean orgs

Revision: 0056
Revision ID: 24537c5bdb0
Revises: 50cea6671951
Create Date: 2016-04-14 19:26:53.721677

"""

# revision identifiers, used by Alembic.
revision = '24537c5bdb0'
down_revision = '50cea6671951'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.execute("DELETE FROM organizations WHERE is_user_org IS TRUE AND created_by_id IS NULL")
    op.execute("""DELETE FROM organizations WHERE id IN (
                SELECT req.id FROM (
                SELECT o.id, COUNT(o.name) \"members\" FROM organizations o
                LEFT OUTER JOIN organization_members om ON o.id = om.org_id
                WHERE o.created_by_id IS NULL AND o.is_user_org IS FALSE
                GROUP BY o.id) req
                WHERE members > 0)""")


def downgrade():
    pass



"""add_automation_role_member

Revision: 0233
Revision ID: 437b60ce2a7c
Revises: 658071592313
Create Date: 2022-03-21 08:06:46.042030

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '437b60ce2a7c'
down_revision = '658071592313'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        INSERT INTO role_permissions (role_id, object_type, actions)
        SELECT roles.id, 'project', '{"RUN_AUTO_ACTIONS"}'::varchar[]
        FROM roles WHERE roles.builtin = 'MEMBER'
    """)

    op.execute("""
        UPDATE roles SET cache_id = uuid_generate_v4()
        WHERE roles.id IN (
            SELECT DISTINCT roles.id
            FROM roles, role_permissions
            WHERE role_permissions.role_id = roles.id
            AND roles.builtin in ('ADMIN', 'MEMBER')
            AND role_permissions.object_type = 'project'
            AND role_permissions.object_id IS NULL
            AND (role_permissions.actions @> '{"RUN_AUTO_ACTIONS"}'::varchar[])
        )
    """)

    op.execute("""
        UPDATE projects SET members_cache_id = uuid_generate_v4() FROM organizations, roles, role_permissions
        WHERE organizations.id = projects.owner_id
            AND roles.owner_id = organizations.id
            AND roles.builtin in ('ADMIN', 'MEMBER')
            AND role_permissions.role_id = roles.id
            AND role_permissions.object_type = 'project'
            AND role_permissions.object_id IS NULL
            AND (role_permissions.actions @> '{"RUN_AUTO_ACTIONS"}'::varchar[])
    """)


def downgrade():
    pass



"""share_objects_unique_index

Revision: 0179
Revision ID: 1b68b8d9f3d6
Revises: 3c5285bbbb66
Create Date: 2020-04-17 10:32:26.948538

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '1b68b8d9f3d6'
down_revision = '3c5285bbbb66'
branch_labels = None
depends_on = None


def upgrade():
    op.drop_index(op.f('ix_share_objects_object_type_object_id'))
    op.create_index(op.f('ix_share_objects_object_type_object_id'), 'share_objects', ['object_type', 'object_id'], unique=True)


def downgrade():
    pass



"""project_image_cover

Revision: 0096
Revision ID: 5397fd6fbb4d
Revises: 19a97b706e5a
Create Date: 2017-06-20 14:58:54.184206

"""

# revision identifiers, used by Alembic.
revision = '5397fd6fbb4d'
down_revision = '19a97b706e5a'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('projects', sa.Column('image_cover_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'projects', 'files', ['image_cover_id'], ['id'], ondelete='SET NULL')


def downgrade():
    op.drop_column('projects', 'image_cover_id')



"""can_create_projects

Revision: 0046
Revision ID: 381538a31a10
Revises: 34d8e77407
Create Date: 2016-02-12 13:52:07.229731

"""

# revision identifiers, used by Alembic.
revision = '381538a31a10'
down_revision = '34d8e77407'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    op.add_column('organizations', sa.Column('can_create_private_projects', sa.Boolean(), nullable=True))
    op.add_column('organizations', sa.Column('can_create_projects', sa.Boolean(), nullable=True))
    op.add_column('users', sa.Column('can_join_organizations', sa.Boolean(), nullable=True))
    op.add_column('users', sa.Column('is_observer', sa.Boolean(), nullable=True))
    op.execute("UPDATE organizations SET can_create_projects = true, can_create_private_projects = can_create_private_boards")
    op.execute("UPDATE users SET can_join_organizations = true, is_observer = false")


def downgrade():
    op.execute("UPDATE organizations SET can_create_private_boards = can_create_private_projects")
    op.drop_column('organizations', 'can_create_projects')
    op.drop_column('organizations', 'can_create_private_projects')
    op.drop_column('users', 'can_join_organizations')
    op.drop_column('users', 'is_observer')


"""Clean group_types

Revision: 0106
Revision ID: 187c2a740494
Revises: 2272df6f8a44
Create Date: 2017-10-10 18:41:07.566598

"""

# revision identifiers, used by Alembic.
revision = '187c2a740494'
down_revision = '2272df6f8a44'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

group_types = sa.Table(
    'card_group_types',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('name', sa.String()),
    sa.Column('attribute_id', sa.Integer)
)

attributes = sa.Table(
    'attributes',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('group_id', sa.Integer)
)

card_models = sa.Table(
    'card_models',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('attribute_group_id', sa.Integer)
)

def upgrade():
    conn = op.get_bind()

    q = sa.select([group_types.c.attribute_id]).where(group_types.c.attribute_id is not None).group_by(group_types.c.attribute_id).having(sa.func.count(group_types.c.attribute_id) > 1)
    q2 = sa.select([attributes.c.group_id]).where(attributes.c.id.in_(q))
    q3 = card_models.select().where(card_models.c.attribute_group_id.in_(q2))
    models = ""
    for model in conn.execute(q3):
        if models != '':
            models += ','
        models += '%s' % model[0]

    q2 = group_types.select().where(group_types.c.attribute_id.in_(q)).order_by(group_types.c.attribute_id, group_types.c.id)
    orig_gt = None
    for gt in conn.execute(q2):
        if orig_gt is None or orig_gt[3] != gt[3]:
            orig_gt = gt
            continue
        op.execute('UPDATE card_groups SET group_type_id =%d, cache_id = uuid_generate_v4() WHERE group_type_id = %d' % (orig_gt[0], gt[0]))
        op.execute('DELETE FROM card_group_types WHERE id = %d' % gt[0])

    if models != "":
        op.execute('UPDATE card_models SET cache_id = uuid_generate_v4() WHERE id IN (%s)' % models)

def downgrade():
    pass



"""show_on_tile

Revision: 0057
Revision ID: 36a48ba6355f
Revises: 24537c5bdb0
Create Date: 2016-05-10 12:45:39.334089

"""

# revision identifiers, used by Alembic.
revision = '36a48ba6355f'
down_revision = '24537c5bdb0'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('card_model_attributes', sa.Column('show_on_tile', sa.Boolean(), nullable=True))
    op.execute("UPDATE card_model_attributes SET show_on_tile = false")
    op.execute("UPDATE card_model_attributes SET show_on_tile = true WHERE type_name IN ('labels', 'group_type', 'files', 'members', 'date', 'indicators', 'size', 'relationships')")


def downgrade():
    op.drop_column('card_model_attributes', 'show_on_tile')



"""user_auto_disable

Revision: 0157
Revision ID: 3068f1397e95
Revises: 3dd85740e8ac
Create Date: 2019-01-30 23:21:08.256457

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '3068f1397e95'
down_revision = '3dd85740e8ac'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('users', sa.Column('is_auto_disabled', sa.Boolean(), nullable=True))
    op.execute("""UPDATE users SET is_auto_disabled = false""")


def downgrade():
    op.drop_column('users', 'is_auto_disabled')



"""request_form_inbound_email

Revision: 0161
Revision ID: 493635f05ca5
Revises: 1ef3e95bf70d
Create Date: 2019-03-12 13:59:32.638392

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '493635f05ca5'
down_revision = '1ef3e95bf70d'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('request_forms', sa.Column('notify_requester_of_group_change', sa.Boolean(), nullable=True))
    op.add_column('request_forms', sa.Column('allow_request_via_email', sa.Boolean(), nullable=True))
    op.add_column('request_forms', sa.Column('email_subject_field_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'request_forms', 'request_form_fields', ['email_subject_field_id'], ['id'])
    op.add_column('request_forms', sa.Column('email_body_field_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'request_forms', 'request_form_fields', ['email_body_field_id'], ['id'])
    op.add_column('request_forms', sa.Column('email_attachments_field_id', sa.Integer(), nullable=True))
    op.create_foreign_key(None, 'request_forms', 'request_form_fields', ['email_attachments_field_id'], ['id'])

    op.execute("UPDATE request_forms SET notify_requester_of_group_change = true, allow_request_via_email = false")
    op.execute("UPDATE request_form_fields SET field_type = 'title' WHERE field_type = 'attribute' AND attribute_id IS NULL")


def downgrade():
    op.drop_column('request_forms', 'allow_request_via_email')
    op.drop_column('request_forms', 'email_subject_field_id')
    op.drop_column('request_forms', 'email_body_field_id')
    op.drop_column('request_forms', 'email_attachments_field_id')



"""org_color

Revision: 0212
Revision ID: 44a1660b8ce7
Revises: f9c853eb5e2c
Create Date: 2021-05-28 18:21:35.012925

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = '44a1660b8ce7'
down_revision = 'f9c853eb5e2c'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column('organizations', sa.Column('color', sa.String(), nullable=True))


def downgrade():
    op.drop_column('organizations', 'color')



"""non-org grouptypes to projects

Revision: 0072
Revision ID: 44182012460b
Revises: 4ed4e16e500e
Create Date: 2016-09-07 15:00:09.809749

"""

# revision identifiers, used by Alembic.
revision = '44182012460b'
down_revision = '4ed4e16e500e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql
import uuid

group_types = sa.Table(
    'card_group_types',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('name', sa.String),
    sa.Column('singular_name', sa.String),
    sa.Column('plural_name', sa.String),
    sa.Column('scope', sa.String),
    sa.Column('one_group_per_card', sa.Boolean),
    sa.Column('display_on_card', sa.String),
    sa.Column('auto_random_color', sa.Boolean),
    sa.Column('builtin', sa.String),
    sa.Column('auto_enabled_on_new_projects', sa.Boolean),
    sa.Column('is_project_specific', sa.Boolean),
    sa.Column('created_by_id', sa.Integer),
    sa.Column('created_at', sa.DateTime),
    sa.Column('updated_at', sa.DateTime),
    sa.Column('icon', sa.String),
    sa.Column('description', sa.Text),
    sa.Column('cache_id', sa.String),
    sa.Column('attribute_id', sa.Integer)
)

project_enabled_group_types = sa.Table(
    'project_enabled_group_types',
    sa.MetaData(),
    sa.Column('group_type_id', sa.Integer()),
    sa.Column('project_id', sa.Integer())
)

card_groups = sa.Table(
    'card_groups',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('parent_id', sa.Integer),
    sa.Column('group_type_id', sa.Integer),
    sa.Column('title', sa.String),
    sa.Column('color', sa.String),
    sa.Column('position', sa.Integer)
)

projects = sa.Table(
    'projects',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('owner_id', sa.Integer),
    sa.Column('top_level_card_id', sa.Integer),
    sa.Column('attribute_group_id', sa.Integer),
    sa.Column('use_org_attributes', sa.Boolean)
)

views = sa.Table(
    'views',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('project_id', sa.Integer),
    sa.Column('group_type_id', sa.Integer)
)

cards = sa.Table(
    'cards',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('project_id', sa.Integer),
    sa.Column('parent_id', sa.Integer),
    sa.Column('board_created', sa.Boolean)
)

def upgrade():
    done = {}
    conn = op.get_bind()

    def copy_gt(gt, project):
        r = conn.execute(group_types.insert().values(
            owner_id=gt.owner_id,
            name=gt.name,
            singular_name=gt.singular_name,
            plural_name=gt.plural_name,
            scope=gt.scope,
            one_group_per_card=gt.one_group_per_card,
            display_on_card=gt.display_on_card,
            auto_random_color=gt.auto_random_color,
            created_by_id=gt.created_by_id,
            created_at=gt.created_at,
            updated_at=gt.updated_at,
            icon=gt.icon,
            description=gt.description,
            builtin=gt.builtin,
            attribute_id=gt.attribute_id,
            cache_id=uuid.uuid4(),
            auto_enabled_on_new_projects=False,
            is_project_specific=True
        ))
        gt_id = r.inserted_primary_key[0]
        conn.execute(card_groups.update().where(sa.and_(card_groups.c.group_type_id == gt.id, card_groups.c.parent_id == project.top_level_card_id)).values(
            group_type_id = gt_id
        ))
        sub_stmt = sa.select([cards.c.id]).where(sa.and_(cards.c.project_id == project.id, cards.c.parent_id != None))
        conn.execute(card_groups.update().where(sa.and_(card_groups.c.group_type_id == gt.id, card_groups.c.parent_id.in_(sub_stmt))).values(
            group_type_id = gt_id
        ))
        return gt_id

    for gt in conn.execute(group_types.select().where(sa.and_(group_types.c.is_project_specific == False, group_types.c.scope != 'org'))):
        if gt.attribute_id is not None:
            continue
        for project_id in conn.execute(project_enabled_group_types.select().where(project_enabled_group_types.c.group_type_id == gt.id)):
            project = conn.execute(projects.select().where(projects.c.id == project_id[1])).first()
            done_id = "{0}-{1}".format(gt.id, project.id)
            if done_id in done:
                continue
            gt_id = copy_gt(gt, project)
            conn.execute(project_enabled_group_types.insert().values(
                group_type_id=gt_id,
                project_id=project.id
            ))
            conn.execute(views.update().where(sa.and_(views.c.group_type_id == gt.id, views.c.project_id == project.id)).values(
                group_type_id = gt_id
            ))
            done[done_id] = True
        op.execute("DELETE FROM card_group_types WHERE id = %s" % gt.id)


def downgrade():
    pass



"""delete_subscriptions_duplicates

Revision: 0105
Revision ID: 2272df6f8a44
Revises: 191bac60ca68
Create Date: 2017-09-28 16:53:10.570392

"""

# revision identifiers, used by Alembic.
revision = '2272df6f8a44'
down_revision = '191bac60ca68'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.execute("""
        delete from card_subscriptions where id in (
            select id from (
                SELECT id, ROW_NUMBER() OVER(PARTITION BY user_id, card_id ORDER BY card_id, user_id) AS n
                FROM card_subscriptions
            ) dups
            where dups.n > 1
        )
    """)


def downgrade():
    pass



"""remove_org_focus

Revision: 0238
Revision ID: f9352a1f06eb
Revises: f026609b2ae2
Create Date: 2022-03-08 11:58:54.082504

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = 'f9352a1f06eb'
down_revision = 'f026609b2ae2'
branch_labels = None
depends_on = None


def upgrade():
    op.execute("""
        UPDATE views SET org_id = null WHERE user_id IS NOT NULL AND project_id IS NULL
    """)

    op.execute("""
        UPDATE users SET app_params = jsonb_set(app_params::jsonb, '{hiddenOrgs}',
            (SELECT jsonb_agg(org_id) FROM organization_members WHERE user_id = users.id AND permission > 0 AND org_id != (app_params->'appContextFocus'->>1)::integer))
        WHERE app_params->'appContextFocus'->>0 = 'org'
    """)

    op.execute("""
        UPDATE users SET app_params = jsonb_set(app_params::jsonb, '{home}', concat('"org:', organizations.slug, '"')::jsonb)
        FROM organizations
        WHERE app_params->'appContextFocus'->>0 = 'org' AND organizations.id = (app_params->'appContextFocus'->>1)::integer
    """)

    op.execute("""
        UPDATE users SET app_params = jsonb_set(app_params::jsonb, '{home}', '"inbox"'::jsonb)
        WHERE app_params->>'inboxAsHome' = 'true'
    """)

    op.execute("""
        UPDATE users SET app_params = jsonb_strip_nulls(jsonb_set(jsonb_set(jsonb_set(jsonb_set(
            app_params::jsonb,
            '{inboxAsHome}', 'null'::jsonb),
            '{last_whatsnew_id_read}', 'null'::jsonb),
            '{last_whatsnew_id_closed}', 'null'::jsonb),
            '{appContextFocus}', 'null'::jsonb)
        )::json
    """)


def downgrade():
    pass
